@@ -202,6 +202,7 @@ val debug : (unit -> unit) -> unit
 val _profile_table : (string, (float ref * int ref)) Hashtbl.t ref
 val profile_code : string -> (unit -> 'a) -> 'a
 val profile_diagnostic : unit -> string
+val reset_profile : unit -> unit
 
 val profile_code_exclusif : string -> (unit -> 'a) -> 'a
 val profile_code_inside_exclusif_ok : string -> (unit -> 'a) -> 'a
@@ -723,6 +724,19 @@ val some : 'a option -> 'a (* alias *)
 val fmap :       ('a -> 'b) -> 'a option -> 'b option
 val map_option : ('a -> 'b) -> 'a option -> 'b option (* alias *)
 
+val equal_option : ('a -> 'b -> bool) -> 'a option -> 'b option -> bool
+(**
+ * [equal_option eq x y] returns [true] iff
+ * either [x = None] and [y = None],
+ * or [x = Some x'] and [y = Some y'] and [eq x' y'].
+ *)
+
+val default : 'a -> ('b -> 'a) -> 'b option -> 'a
+(**
+ * [default d f None] returns [d]
+ * and [default d f (Some x)] returns [f x]
+ *)
+
 val do_option : ('a -> unit) -> 'a option -> unit
 
 val optionise : (unit -> 'a) -> 'a option
@@ -735,7 +749,19 @@ val partition_either3 :
     ('a -> ('b, 'c, 'd) either3) -> 'a list -> 'b list * 'c list * 'd list
 
 val filter_some : 'a option list -> 'a list
+(**
+ * [filter_some [None;Some x1; None; Some X2; ... Some xn;None]]
+ * Returns [[x1;x2;...;xn]]
+ *)
+
 val map_filter : ('a -> 'b option) -> 'a list -> 'b list
+(**
+ * [map_filter f [x1;...;xn]] returns the list of results obtained
+ * by applying f to x1 ... xn, given that not all applications of f
+ * may lead to a result. So if [f xi] is None it does not appear in the
+ * resulting list, but if it's [Some yi] then [yi] is in the returned list.
+ *)
+
 val tail_map_filter : ('a -> 'b option) -> 'a list -> 'b list
 val find_some : ('a -> 'b option) -> 'a list -> 'b
 
@@ -792,6 +818,7 @@ val md5sum_of_string : string -> string
 (*****************************************************************************)
 
 val regexp_alpha : Str.regexp
+val regexp_int : Str.regexp
 val regexp_word : Str.regexp
 
 val _memo_compiled_regexp : (string, Str.regexp) Hashtbl.t
@@ -995,8 +1022,6 @@ val interpolate : string -> string list
 
 val echo : string -> string
 
-val usleep : int -> unit
-
 val process_output_to_list : string -> string list
 val cmd_to_list :            string -> string list (* alias *)
 val cmd_to_list_and_status : string -> string list * Unix.process_status
@@ -1013,7 +1038,6 @@ val do_in_fork : (unit -> unit) -> int
 
 val mkdir: ?mode:Unix.file_perm -> string -> unit
 
-val read_file : filename -> string
 val write_file : file:filename -> string -> unit
 
 val filesize : filename -> int
@@ -1071,9 +1095,9 @@ exception Timeout
  *
  * The same is true for UnixExit (see below).
  *)
-val timeout_function : int -> (unit -> 'a) -> 'a
+val timeout_function : string -> int -> (unit -> 'a) -> 'a
 
-val timeout_function_opt : int option -> (unit -> 'a) -> 'a
+val timeout_function_opt : string -> int option -> (unit -> 'a) -> 'a
 
 val remove_file : string -> unit
 
@@ -1283,6 +1307,12 @@ val sorted_keep_best : ('a -> 'a -> 'a option) -> 'a list -> 'a list
 
 val cartesian_product : 'a list -> 'b list -> ('a * 'b) list
 
+val equal_list : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool
+(**
+ * [equal_list eq [x0; x1; ...; xm] [y0; y1; ...; yn]] returns [true] iff
+ * [m = n] and [eq x0 y0], [eq x1 y1], ..., and [eq xn yn].
+ *)
+
 (* old stuff *)
 val surEnsemble : 'a list -> 'a list list -> 'a list list
 val realCombinaison : 'a list -> 'a list list
@@ -1949,3 +1979,5 @@ class ['a] olist :
   end
 
 val typing_sux_test : unit -> unit
+
+module StringSet: Set.S with type elt = string
