@@ -44,7 +44,7 @@ let contains_modif used_after x =
       V.combiner bind option_default
 	mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
 	mcode mcode mcode
-	do_nothing do_nothing do_nothing do_nothing do_nothing
+	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
 	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
 	do_nothing do_nothing do_nothing do_nothing do_nothing
 	do_nothing rule_elem do_nothing do_nothing do_nothing do_nothing in
@@ -72,7 +72,7 @@ let contains_constant x =
 	  mcode mcode mcode
 	  do_nothing do_nothing do_nothing do_nothing do_nothing
 	  ident expr do_nothing do_nothing do_nothing do_nothing
-	  do_nothing do_nothing do_nothing do_nothing do_nothing
+	  do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
 	  do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing in
       recursor.V.combiner_rule_elem x
   | _ -> true
@@ -135,7 +135,7 @@ let strip x =
       mcode mcode mcode mcode mcode
       do_nothing do_nothing do_nothing do_nothing do_nothing
       do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-      do_nothing do_nothing do_nothing do_nothing
+      do_nothing do_nothing do_nothing do_nothing do_nothing
       decl do_absolutely_nothing rule_elem do_nothing do_nothing
       do_nothing do_absolutely_nothing in
   recursor.V.rebuilder_rule_elem x
@@ -144,7 +144,7 @@ let strip x =
 
 let disj l1 l2 = l1 l2
 
-let rec conj xs ys =
+let conj xs ys =
   match (xs,ys) with
     ([],_) -> ys
   | (_,[]) -> xs
@@ -168,7 +168,7 @@ let conj_wrapped x l = conj [List.map (function x -> (1,strip x)) x] l
 (* --------------------------------------------------------------------- *)
 (* the main translation loop *)
 
-let rec rule_elem re =
+let rule_elem re =
   match Ast.unwrap re with
     Ast.DisjRuleElem(res) ->
       (* why was the following done?  ors have to be kept together for
