@@ -312,16 +312,16 @@ module XMATCH = struct
 
 
   (* ------------------------------------------------------------------------*)
-  (* Constraints on metavariable values *)
+  (* Constraints on position metavariables *)
   (* ------------------------------------------------------------------------*)
 
-  let check_constraints_ne matcher constraints exp = fun f tin ->
+ let check_constraints_ne matcher constraints exp = fun f tin ->
     let rec loop = function
-	[] -> f () tin (* success *)
-      |	c::cs ->
-	  match matcher c exp tin with
-	    [] (* failure *) -> loop cs
-	  | _ (* success *) -> fail tin in
+        [] -> f () tin (* success *)
+      | c::cs ->
+          match matcher c exp tin with
+            [] (* failure *) -> loop cs
+          | _ (* success *) -> fail tin in
     loop constraints
 
   let check_constraints matcher constraints term = fun f tin ->
@@ -329,22 +329,11 @@ module XMATCH = struct
     then f () tin (* success *)
     else fail tin (* failure *)
 
-  let check_pos_constraints constraints pvalu f tin =
-    check_constraints_ne
-      (fun c exp tin ->
-	let success = [[]] in
-	let failure = [] in
-	(* relies on the fact that constraints on pos variables must refer to
-	   inherited variables *)
-	(match Common.optionise (fun () -> tin.binding0 +> List.assoc c) with
-	  Some valu' ->
-	    if Cocci_vs_c.equal_inh_metavarval exp valu'
-	    then success else failure
-	| None ->
-	    (* if the variable is not there, it puts no constraints *)
-	    (* not sure this is still useful *)
-	    failure))
-      constraints pvalu f tin
+  let check_pos_constraints pname constraints pvalu f tin =
+    let res =
+      Cocci_vs_c.satisfies_constraint constraints (pname, pvalu)
+	(fun name -> List.assoc name tin.binding0) in
+    if res then f () tin (* success *) else fail tin (* failure *)
 
   (* ------------------------------------------------------------------------*)
   (* Environment *)
@@ -486,6 +475,13 @@ module XMATCH = struct
 		   then Lib_parsing_c.al_params a
 		   else Lib_parsing_c.semi_al_params a))
 
+          | Ast_c.MetaDParamListVal a ->
+	      success
+		(Ast_c.MetaDParamListVal
+		   (if strip
+		   then Lib_parsing_c.al_define_params a
+		   else Lib_parsing_c.semi_al_define_params a))
+
           | Ast_c.MetaFragListVal a ->
 	      success
 		(Ast_c.MetaFragListVal
@@ -512,7 +508,8 @@ module XMATCH = struct
 	let rec loop tin = function
 	    [] -> finish tin
 	  | Ast_cocci.MetaPos(name,constraints,per,keep,inherited) :: rest ->
-	      check_pos_constraints constraints pvalu
+	      let name' = Ast_cocci.unwrap_mcode name in
+	      check_pos_constraints name' constraints pvalu
 		(function () ->
 	    (* constraints are satisfied, now see if we are compatible
 	       with existing bindings *)
