@@ -32,7 +32,8 @@ let patch = ref (None : string option)
 let make_hrule = ref (None : string (*dir*) option)
 let hrule_per_file = ref true (* if false, then a rule per function *)
 
-let currentfile = ref (None : string option)
+let currentfile = ref (None : string option) (* file of current code *)
+let currentfiles = ref ([] : string list) (* starting files of this run *)
 
 let current_element = ref ""
 let dir = ref ""
@@ -55,3 +56,8 @@ let ibm = ref false
 let include_headers = ref false
 
 exception UnreadableFile of string
+
+let cocci_attribute_names = ref ([] : string list)
+let add_cocci_attribute_names s =
+  if not (List.mem s !cocci_attribute_names)
+  then cocci_attribute_names := s :: !cocci_attribute_names
