@@ -6,6 +6,7 @@
 
 open Common
 module FC = Flag_cocci
+module Inc = Includes
 
 (*****************************************************************************)
 (* Flags *)
@@ -18,7 +19,8 @@ module FC = Flag_cocci
 
 let cocci_file = ref ""
 
-let output_file = ref ""
+let output_file = ref "" (* resulting code *)
+let tmp_dir = ref "" (* temporary files for parallelism *)
 let inplace_modif = ref false  (* but keeps nothing *)
 let backup_suffix =
   ref (None : string option) (* suffix for backup if one is desired *)
@@ -26,10 +28,10 @@ let outplace_modif = ref false (* generates a .cocci_res  *)
 let preprocess = ref false     (* run the C preprocessor before cocci *)
 let compat_mode = ref false
 let ignore_unknown_opt = ref false
+let profile_per_file = ref false
 
-(* somehow obsolete now *)
 let dir = ref false
-
+let file_groups = ref false
 let kbuild_info = ref ""
 
 let macro_file = ref ""
@@ -253,6 +255,23 @@ let long_usage_func  = ref (fun () -> ())
 (* will be printed when use only ./spatch. For the rest you have to
  * use -longhelp to see them.
  *)
+
+let print_version () =
+  let withpython = if Pycocci.python_support then "yes" else "no" in
+  let whichregexp =
+    if !Regexp.pcre_support then "PCRE"
+    else "Str" in
+  let flags =
+    if Config.configure_flags<>"" then Config.configure_flags
+    else "[none]" in
+  (* let mode = if Dynlink.is_native then "native" else "byte-code" in *)
+  Printf.printf "spatch version %s compiled with OCaml version %s\n"
+    (* mode *) Config.version Config.ocaml_version;
+  Printf.printf "Flags passed to the configure script: %s\n" flags;
+  Printf.printf "Python scripting support: %s\n" withpython;
+  Printf.printf "Syntax of regular expresssions: %s\n" whichregexp;
+  exit 0
+
 let short_options = [
   "--sp-file",  Arg.Set_string cocci_file,
   " <file> the semantic patch file";
@@ -281,26 +300,28 @@ let short_options = [
   " <file> (default=" ^ !Config.std_h ^ ")";
 
   "--recursive-includes",
-  Arg.Unit (function _ -> FC.include_options := FC.I_REALLY_ALL_INCLUDES),
-  "  causes all available include files, both those included in the C file(s) and those included in header files, to be used";
+  Arg.Unit (function _ -> Inc.set_parsing_style Inc.Parse_really_all_includes),
+  ("  causes all available include files, both those included in the"^
+   " C file(s) and those included in header files, to be used");
   "--all-includes",
-  Arg.Unit (function _ -> FC.include_options := FC.I_ALL_INCLUDES),
+  Arg.Unit (function _ -> Inc.set_parsing_style Inc.Parse_all_includes),
   "  causes all available include files included in the C file(s) to be used";
   "--no-includes",
-  Arg.Unit (function _ -> FC.include_options := FC.I_NO_INCLUDES),
+  Arg.Unit (function _ -> Inc.set_parsing_style Inc.Parse_no_includes),
   "  causes not even local include files to be used";
   "--local-includes",
-  Arg.Unit (function _ -> FC.include_options := FC.I_NORMAL_INCLUDES),
+  Arg.Unit (function _ -> Inc.set_parsing_style Inc.Parse_local_includes),
   "  causes local include files to be used";
-  "--include-headers-for-types", Arg.Set FC.include_headers_for_types,
+  "--include-headers-for-types", Arg.Set Inc.include_headers_for_types,
   "    use only type information from header files";
   "--ignore-unknown-options", Arg.Set ignore_unknown_opt,
-  "    For integration in a toolchain (must be set before the first unknown option)";
+  ("    For integration in a toolchain (must be set before the first unknown"^
+   " option)");
   "--include-headers", Arg.Set Flag.include_headers,
   "    process header files independently";
-  "-I",   Arg.String (fun x -> FC.include_path:= x::!FC.include_path),
+  "-I",   Arg.String (fun x -> Inc.include_path:= x::!Inc.include_path),
   "  <dir> containing the header files";
-  "--include", Arg.String (fun x -> FC.extra_includes:=x::!FC.extra_includes),
+  "--include", Arg.String (fun x -> Inc.extra_includes:=x::!Inc.extra_includes),
   "  file to consider as being included";
 
   "--preprocess", Arg.Set preprocess,
@@ -310,6 +331,8 @@ let short_options = [
 
   "--dir", Arg.Set dir,
   "    <dir> process all files in directory recursively";
+  "--file-groups", Arg.Set file_groups,
+  "    <file> process the file groups listed in the file";
 
   "--no-scanner", Arg.Unit (function _ -> Flag.scanner := Flag.NoScanner),
   "    no indexing";
@@ -341,17 +364,7 @@ let short_options = [
   "--print-options-only", Arg.Unit (fun () -> ()),
   "   print selected options and exit";
 
-  "--version",   Arg.Unit (fun () ->
-    let withpython = if Pycocci.python_support then "with" else "without" in
-    let whichregexp =
-      if !Regexp.pcre_support then "with PCRE support"
-      else "with Str regexp support "
-    in
-    Printf.printf "spatch version %s %s Python support and %s\n"
-      Config.version withpython whichregexp;
-    exit 0;
-  ),
-    "  guess what";
+  "--version",   Arg.Unit print_version, "  print version and build info";
 
   "--date",   Arg.Unit (fun () ->
     Printf.printf "version: $Date$";
@@ -497,6 +510,10 @@ let other_options = [
   [
     "--profile", Arg.Unit (function () -> Common.profile := Common.PALL) ,
     "   gather timing information about the main coccinelle functions";
+    "--profile-per-file",
+    Arg.Unit
+    (function () -> profile_per_file :=true; Common.profile := Common.PALL),
+    "   gather timing information for each file (implies --profile)";
     "--bench", Arg.Int (function x -> Flag_ctl.bench := x),
     "   <level> for profiling the CTL engine";
     "--timeout",
@@ -544,7 +561,7 @@ let other_options = [
 
     "--hrule", Arg.String
     (function s ->
-      Flag.make_hrule := Some s; FC.include_options := FC.I_NO_INCLUDES),
+      Flag.make_hrule := Some s; Inc..include_options := Inc.Parse_no_includes),
     "    semantic patch generation";
 *)
     "--keep-comments", Arg.Set Flag_parsing_c.keep_comments,
@@ -646,6 +663,8 @@ let other_options = [
     "   the number of cores to be used";
     "--chunksize", Arg.Int (function x -> parmap_chunk_size := Some x),
     "   the size of work chunks for parallelism";
+    "--tmp-dir", Arg.Set_string tmp_dir,
+    "   prefix of temporary directories for parallelism";
   ];
 
   "pad options",
@@ -691,7 +710,7 @@ let other_options = [
     "   which score file to compare with in --test-spacing";
     "--no-update-score-file", Arg.Clear allow_update_score_file,
     "   do not update the score file when -testall succeeds";
-    "--relax-include-path", Arg.Set FC.relax_include_path,
+    "--relax-include-path", Arg.Set Inc.relax_include_path,
     " ";
   ];
 
@@ -741,7 +760,7 @@ let check_include_path () =
   let is_include_re = Str.regexp "-I\\(.*\\)" in
   if Str.string_match is_include_re opt 0 then
     let path = Str.matched_group 1 opt in
-	FC.include_path:= path::!FC.include_path
+	Inc.include_path:= path::!Inc.include_path
   else ()
 
 let rec arg_parse_no_fail l f msg =
@@ -827,14 +846,17 @@ let glimpse_filter2 (_,query,_,_) dir =
 		Some
 		  (let filelist = glimpse_res +>
 		   List.filter
-		     (fun file -> List.mem (Common.filesuffix file) suffixes) in
+		     (fun file ->
+		       List.mem (Common.filesuffix file) suffixes) in
 		   if filelist <> [] then
 		     begin
 		       let firstfile = List.hd filelist in
-		       if Filename.is_relative firstfile || Filename.is_implicit firstfile then
-			 List.map (fun file -> dir ^ Filename.dir_sep ^ file) filelist
-		       else
-			 filelist
+		       if Filename.is_relative firstfile ||
+		          Filename.is_implicit firstfile
+		       then
+			 List.map (fun file -> dir ^ Filename.dir_sep ^ file)
+			   filelist
+		       else filelist
 		     end
 		   else []
 		  )
@@ -866,14 +888,55 @@ let gitgrep_filter ((_,_,query,_) as x) dir =
       |	None -> coccigrep_filter x dir
 
 let idutils_filter (_,_,_,query) dir =
-  match query with
-    None -> pr2 "no inferred idutils keywords"; None
-  | Some query ->
+  match Id_utils.interpret dir query with
+    None -> None
+  | Some files ->
       let suffixes = if !Flag.include_headers then ["c";"h"] else ["c"] in
-      let files = Id_utils.interpret dir query in
+      Printf.eprintf "%d files match\n" (List.length files);
       Some
 	(files +>
-	 List.filter (fun file -> List.mem (Common.filesuffix file) suffixes))
+	 List.filter
+	   (fun file -> List.mem (Common.filesuffix file) suffixes))
+
+let scanner_to_interpreter = function
+    Flag.Glimpse -> glimpse_filter
+  | Flag.IdUtils -> idutils_filter
+  | Flag.CocciGrep -> coccigrep_filter
+  | Flag.GitGrep -> gitgrep_filter
+  | _ -> failwith "impossible"
+
+(*****************************************************************************)
+(* File groups *)
+(*****************************************************************************)
+
+(* Groups are separated by blank lines. Single-line comments beginning
+with // are tolerated *)
+let read_file_groups x =
+  let i = open_in x in
+  let file_groups = ref [] in
+  let res = ref [] in
+  let dump _ =
+    (match !res with
+      [] -> ()
+    | l -> file_groups := (List.rev l) :: !file_groups);
+    res := [] in
+  let empty_line = Str.regexp " *$" in
+  let com = Str.regexp " *//" in
+  let rec in_files _ =
+    let l = input_line i in
+    if Str.string_match empty_line l 0
+    then begin dump(); in_space() end
+    else if Str.string_match com l 0
+    then in_files()
+    else begin res := l :: !res; in_files() end
+  and in_space _ =
+    let l = input_line i in
+    if Str.string_match empty_line l 0
+    then in_space()
+    else if Str.string_match com l 0
+    then in_space()
+    else begin res := l :: !res; in_files() end in
+  try in_files() with End_of_file -> begin dump(); List.rev !file_groups end
 
 (*****************************************************************************)
 (* Main action *)
@@ -902,49 +965,64 @@ let rec main_action xs =
 
 	  if !dir && !Flag.patch = None
 	  then
-	    (match xs with
-	    | [] -> Flag.patch := Some (Cocci.normalize_path x)
-	    | _ ->
-		pr2
-		  ("warning: patch output can only be created when only one\n"^
-		   "directory is specified or when the -patch flag is used")
-		  );
-	  Flag.dir := x;
+	    begin
+	      (match xs with
+	      | [] -> Flag.patch := Some (Cocci.normalize_path x)
+	      | _ ->
+		  pr2
+		    ("warning: patch output can only be created when only one\n"
+		     ^"directory is specified or when the -patch flag is used")
+		    );
+	      Flag.dir := x
+	    end;
 
 	let (cocci_infos,constants) =
 	  Cocci.pre_engine (!cocci_file, !Config.std_iso) in
 
         let infiles =
             Common.profile_code "Main.infiles computation" (fun () ->
-	      match !dir, !kbuild_info, !Flag.scanner, xs with
+	      match !file_groups, !dir, !kbuild_info, !Flag.scanner, xs with
             (* glimpse *)
-              | false, _, _, _ -> [x::xs]
-	      |	true, "",
+	      | true, false, "", scanner, [] ->
+		  let fg = read_file_groups x in
+		  (match scanner with
+		    Flag.NoScanner -> fg
+		  | _ ->
+		      let interpreter = scanner_to_interpreter !Flag.scanner in
+		      List.filter
+			(function files ->
+			  List.exists
+			    (function fl ->
+			      match interpreter constants fl with
+				Some [] -> false (* no file matches *)
+			      | _ -> true)
+			    files)
+			fg)
+	      | true, _, _, _, _ ->
+		  failwith
+		    ("file groups not compatible with --dir, --kbuild-info,"^
+		     " or multiple files")
+              | _, false, _, _, _ -> [x::xs]
+	      |	_, true, "",
 		  (Flag.Glimpse|Flag.IdUtils|Flag.CocciGrep|Flag.GitGrep),
 		  [] ->
-		    let interpreter =
-		      match !Flag.scanner with
-			Flag.Glimpse -> glimpse_filter
-		      | Flag.IdUtils -> idutils_filter
-		      | Flag.CocciGrep -> coccigrep_filter
-		      | Flag.GitGrep -> gitgrep_filter
-		      |	_ -> failwith "impossible" in
+		    let interpreter = scanner_to_interpreter !Flag.scanner in
 		    let files =
 		      match interpreter constants x with
 			None -> Test_parsing_c.get_files x
 		      | Some files -> files in
                     files +> List.map (fun x -> [x])
-              | true, s,
+              | _, true, s,
 		  (Flag.Glimpse|Flag.IdUtils|Flag.CocciGrep|Flag.GitGrep), _
 		when s <> "" ->
                   failwith "--use-xxx filters do not work with --kbuild"
                   (* normal *)
-	      | true, "", _, _ ->
+	      | _, true, "", _, _ ->
 		  Test_parsing_c.get_files
 		    (String.concat " " (x::xs)) +> List.map (fun x -> [x])
 
             (* kbuild *)
-	      | true, kbuild_info_file,_,_ ->
+	      | _, true, kbuild_info_file,_,_ ->
 		  let dirs =
                     Common.cmd_to_list
 		      ("find "^(String.concat " " (x::xs))^" -type d") in
@@ -1007,9 +1085,16 @@ let rec main_action xs =
 	    List.fold_left op z l in
 	  let par_fold merge op z l =
             let prefix =
-	      Filename.chop_extension (Filename.basename !cocci_file) in
+	      let prefix =
+		Filename.chop_extension (Filename.basename !cocci_file) in
+	      if !tmp_dir = ""
+	      then prefix
+	      else Printf.sprintf "%s_%s" !tmp_dir prefix in
 	    (if Sys.file_exists prefix
-	    then failwith (Printf.sprintf "Directory %s used for temporary files already exists and should be removed." prefix));
+	    then
+	      failwith
+		(Printf.sprintf
+		   "Directory %s used for temporary files already exists and should be removed." prefix));
 	    let clean _ =
 	      let files = Array.to_list(Sys.readdir prefix) in
 	      let (stdouts,stderrs) =
@@ -1022,16 +1107,26 @@ let rec main_action xs =
 		stderrs;
 	      let _ = Sys.command (Printf.sprintf "rm -rf %s" prefix) in
 	      () in
-	    let res =
+	    let op y (x, temp_files) =
+	      (* The subprocess has a different address space, so its
+		 collected temporary files aren't seen by the main spatch
+		 process *)
+	      Common._temp_files_created := temp_files;
+	      let res = op x y in
+	      (res, !Common._temp_files_created) in
+	    let merge (x, temp_files) (y, temp_files') =
+	      (merge x y, List.rev_append temp_files temp_files') in
+	    let (res, tmps) =
 	      try
 		Parmap.parfold
 		  ~init:(fun id -> Parmap.redirect ~path:prefix ~id)
 		  ~ncores
 		  ~chunksize
-		  (fun x y -> op y x) (Parmap.L l) z merge
+		  op (Parmap.L l) (z, !Common._temp_files_created) merge
 	      with e ->
 		(Printf.eprintf "exception on %s: %s\n" prefix (Dumper.dump e);
 		 clean(); raise e) in
+	    Common._temp_files_created := tmps;
 	    clean();
 	    res
 	  in
@@ -1049,45 +1144,57 @@ let rec main_action xs =
           let outfiles =
             Common.profile_code "Main.outfiles computation" (fun () ->
 	      let res =
-		infiles +> actual_fold (@) (fun prev cfiles ->
-		  if (not !Flag.worth_trying_opt) ||
-		    Cocci.worth_trying cfiles constants
+		match infiles with
+		  [] ->
+		    (* parmap case does a lot of work that is not needed
+		       if there is nothing to do *)
+		    []
+		| _ ->
+		    infiles +> actual_fold (@) (fun prev cfiles ->
+		      if (not !Flag.worth_trying_opt) ||
+		      Cocci.worth_trying cfiles constants
 		      then
-		    begin
-		  pr2 ("HANDLING: " ^ (String.concat " " cfiles));
-		  (*pr2 (List.hd(Common.cmd_to_list "free -m | grep Mem"));*)
-		  flush stderr;
+			begin
+			  pr2 ("HANDLING: " ^ (String.concat " " cfiles));
+			  flush stderr;
 
-		  Common.timeout_function_opt !FC.timeout (fun () ->
-		    Common.report_if_take_time 10 (String.concat " " cfiles)
-		      (fun () ->
-                      try
-			let optfile =
-			  if !output_file <> "" && !compat_mode then
-			    Some !output_file
-			  else
-			    None
-			in
-			List.rev
-			(adjust_stdin cfiles (fun () ->
-			  Common.redirect_stdout_opt optfile (fun () ->
-			  (* this is the main call *)
-			    Cocci.full_engine cocci_infos cfiles
-			))) @ prev
-		      with
-		      | Common.UnixExit x -> raise (Common.UnixExit x)
-		      |	Pycocci.Pycocciexception ->
-			  raise Pycocci.Pycocciexception
-		      | e ->
-			  if !dir
-			  then begin
-			    pr2 ("EXN:" ^ Printexc.to_string e);
-			    prev (* *)
-			  end
-			  else raise e))
+			  let all_cfiles = String.concat " " cfiles in
+			  Common.timeout_function_opt all_cfiles !FC.timeout
+			    (fun () ->
+			      let res =
+				Common.report_if_take_time 10 all_cfiles
+				  (fun () ->
+				    try
+				      let optfile =
+					if !output_file <> "" && !compat_mode
+					then Some !output_file
+					else None in
+				      List.rev
+					(adjust_stdin cfiles (fun () ->
+					  Common.redirect_stdout_opt optfile
+					    (fun () ->
+					      (* this is the main call *)
+					      Cocci.full_engine cocci_infos
+						cfiles
+					 ))) @ prev
+				    with
+				    | Common.UnixExit x ->
+					raise (Common.UnixExit x)
+				    | Pycocci.Pycocciexception ->
+					raise Pycocci.Pycocciexception
+				    | e ->
+					if !dir
+					then begin
+					  pr2 ("EXN:" ^ Printexc.to_string e);
+					  prev (* *)
+					end
+					else raise e) in
+			      (if !dir && !profile_per_file
+			      then Common.reset_profile());
+			      res)
 			end
-		  else prev)
-	      [] in res) in
+		      else prev)
+		      [] in res) in
 	  let outfiles = List.rev outfiles in
 	  (match Iteration.get_pending_instance() with
 	    None ->
@@ -1194,7 +1301,21 @@ let rec fix_idutils = function
   | "--use-idutils"::second::rest
     when String.length second > 0 && String.get second 0 = '-' ->
        "--use-idutils"::".id-utils.index" :: (fix_idutils (second :: rest))
-  | x :: rest -> x :: (fix_idutils rest)
+  | x :: rest ->
+      let fail _ = x :: (fix_idutils rest) in
+      let len = String.length x in
+      if len > 3 && String.get x 0 = '-' && String.get x 1 = '-'
+      then
+	let c1 = String.get x 2 in
+	let s = String.sub x 3 (len-3) in
+	if c1 = 'j'
+	then
+	  try let _ = int_of_string s in "--j"::s::(fix_idutils rest)
+	  with _ -> fail()
+	else if c1 = 'I'
+	then "--I"::s::(fix_idutils rest)
+	else fail()
+      else fail()
 
 (*****************************************************************************)
 (* The coccinelle main entry point *)
@@ -1253,18 +1374,28 @@ let main () =
     (if !dir
     then
       let chosen_dir =
-            if List.length !args > 1
-            then
-              begin
-                let chosen = List.hd !args in
-                Flag.dir := chosen;
-                pr2 ("ignoring all but the last specified directory: "^chosen);
-                args := [chosen];
-                chosen
-              end
-            else List.hd !args
-        in if !FC.include_path = []
-           then FC.include_path := [Filename.concat chosen_dir "include"]);
+        if List.length !args > 1
+        then
+          begin
+            let chosen = List.hd !args in
+            Flag.dir := chosen;
+            pr2 ("ignoring all but the last specified directory: "^chosen);
+            args := [chosen];
+            chosen
+          end
+        else List.hd !args
+      in if !Inc.include_path = []
+      then Inc.include_path := [Filename.concat chosen_dir "include"]);
+    (* The same thing for file groups *)
+    (if !file_groups
+    then
+      if List.length !args > 1
+      then
+        begin
+          let chosen = List.hd !args in
+          pr2 ("ignoring all but the last specified file: "^chosen);
+          args := [chosen]
+        end);
 
     args := List.rev !args;
 
@@ -1307,33 +1438,33 @@ let main () =
 	  let testfile = x ^ ".cocci" in
 	    if Sys.file_exists (prefix ^ testfile) then
 	      begin
-		(if !FC.include_path = []
-		then FC.include_path := [prefix^"include"]);
+		(if !Inc.include_path = []
+		then Inc.include_path := [prefix^"include"]);
 		Testing.testone prefix x !compare_with_expected
 	      end
 	    else
 	      if Sys.file_exists testfile then
 	      begin
-		(if !FC.include_path = []
-		then FC.include_path := ["include"]);
+		(if !Inc.include_path = []
+		then Inc.include_path := ["include"]);
 		Testing.testone "" x !compare_with_expected
 	      end
 	      else
 	        Printf.fprintf stderr
 		  "ERROR: File %s does not exist\n" testfile
-		end
+	end
 
     | []  when !test_all ->
-        (if !FC.include_path = []
-         then FC.include_path := ["tests/include"]);
+        (if !Inc.include_path = []
+         then Inc.include_path := ["tests/include"]);
         let score_file = if !expected_score_file <> ""
                          then !expected_score_file
                          else "tests/SCORE_expected.sexp" in
         Testing.testall score_file !allow_update_score_file
 
     | []  when !test_spacing ->
-        (if !FC.include_path = []
-         then FC.include_path := ["tests/include"]);
+        (if !Inc.include_path = []
+         then Inc.include_path := ["tests/include"]);
         let score_file = if !expected_spacing_score_file <> ""
                          then !expected_spacing_score_file
                          else "tests/SCORE_spacing_expected.sexp" in
@@ -1344,7 +1475,7 @@ let main () =
 
     | ((x::xs) as cfiles) when !test_okfailed ->
         (* do its own timeout on FC.timeout internally *)
-        FC.relax_include_path := true;
+        Inc.for_tests := true;
         adjust_stdin cfiles (fun () ->
           Testing.test_okfailed !cocci_file cfiles
           )
@@ -1357,6 +1488,7 @@ let main () =
         Common.do_action !action xs all_actions
 
     | [] when !action = "--parse-cocci" ->
+	Iso_pattern.verbose_iso := true;
         Testing.test_parse_cocci !cocci_file
 
     | [] when !action = "--rule-dependencies" ->
@@ -1381,8 +1513,8 @@ let main () =
     (* --------------------------------------------------------- *)
     | [] -> short_usage()
     ));
-    if !Pycocci.initialised && (Pycocci.py_isinitialized ()) != 0 then begin
-      ignore(Pycocci.pyrun_simplestring "cocci.finalise()");
+    if !Pycocci.initialised && Pycocci.py_isinitialized () then begin
+      Pycocci.pyrun_simplestring "cocci.finalise()";
       if !Flag.show_misc
       then Common.pr2 "Finalizing python\n";
       Pycocci.py_finalize ();
@@ -1403,6 +1535,10 @@ let main_with_better_error_report () =
     | Parse_cocci.Bad_virt s ->
 	Printf.fprintf stderr "virtual rule %s not supported\n" s;
         raise (UnixExit (-1))
+    | Parse_cocci.SMPLParseError error_message
+    | Failure error_message ->
+	Printf.fprintf stderr "%s\n" error_message;
+        raise (UnixExit (-1))
 
 (*****************************************************************************)
 let start =
