@@ -90,6 +90,7 @@ let ast_rep_binding ctr = function
 	(Printf.sprintf "%s: No AST representation for listlen variables" nm)
   | (Some nm,Ast.MetaParamDecl _) -> print_match ctr nm "Param"
   | (Some nm,Ast.MetaParamListDecl _) -> print_match ctr nm "ParamList"
+  | (Some nm,Ast.MetaDParamListDecl _) -> print_match ctr nm "IdentList"
   | (Some nm,Ast.MetaBinaryOperatorDecl _) ->
       failwith
 	(Printf.sprintf "%s: No AST representation for operator variables" nm)
@@ -139,40 +140,40 @@ let print_iteration_code o =
     match !Iteration.parsed_virtual_rules with
       [] -> ""
     | l ->
-	Printf.sprintf "
-    method add_virtual_rule r =
-      let r = match r with %s in
-      virtual_rules <- Common.union_set [r] virtual_rules\n"
+	Printf.sprintf "\
+\n    method add_virtual_rule r =\
+\n      let r = match r with %s in\
+\n      virtual_rules <- Common.union_set [r] virtual_rules\n"
 	  (translator l) in
   let add_virt_ids_method =
     match !Iteration.parsed_virtual_identifiers with
       [] -> ""
     | l ->
-	Printf.sprintf "
-    method add_virtual_identifier i v =
-      let i = match i with %s in
-      try
-	let v1 = List.assoc i virtual_identifiers in
-	if not (v = v1)
-	then failwith (\"multiple values specified for \"^i)
-      with Not_found ->
-	virtual_identifiers <- (i,v) :: virtual_identifiers"
+	Printf.sprintf "\
+\n    method add_virtual_identifier i v =\
+\n      let i = match i with %s in\
+\n      try\
+\n	let v1 = List.assoc i virtual_identifiers in\
+\n	if not (v = v1)\
+\n	then failwith (\"multiple values specified for \"^i)\
+\n      with Not_found ->\
+\n	virtual_identifiers <- (i,v) :: virtual_identifiers"
 					 (translator l) in
-    Printf.fprintf o "
-class iteration () =
-  object
-    val mutable files = None
-    val mutable files_changed = false
-    val mutable virtual_rules = ([] : string list)
-    val mutable virtual_identifiers = ([] : (string * string) list)
-    val mutable extend_virtual_ids = false
-    method set_files f = files <- Some f
-    method extend_virtual_identifiers (x : unit) = extend_virtual_ids <- true
-    %s%s
-    method register () =
-      Iteration.add_pending_instance
-	(files,virtual_rules,virtual_identifiers,extend_virtual_ids)
-  end\n\n" add_virt_rules_method add_virt_ids_method
+    Printf.fprintf o "\
+\nclass iteration () =\
+\n  object\
+\n    val mutable files = None\
+\n    val mutable files_changed = false\
+\n    val mutable virtual_rules = ([] : string list)\
+\n    val mutable virtual_identifiers = ([] : (string * string) list)\
+\n    val mutable extend_virtual_ids = false\
+\n    method set_files f = files <- Some f\
+\n    method extend_virtual_identifiers (x : unit) = extend_virtual_ids <- true\
+\n    %s%s\
+\n    method register () =\
+\n      Iteration.add_pending_instance\
+\n	(files,virtual_rules,virtual_identifiers,extend_virtual_ids)\
+\n  end\n\n" add_virt_rules_method add_virt_ids_method
 
 (* ---------------------------------------------------------------------- *)
 
@@ -205,11 +206,11 @@ let prepare_mvs o str = function
 	true
       with Not_found -> false
 
-let prepare_rule (name, metavars, script_vars, code) =
+let prepare_generic_rule (name, metavars, script_vars, code) scriptargs fcts =
   let fname = String.concat "_" (Str.split (Str.regexp " ") name) in
   (* function header *)
   let function_header body =
-    Printf.sprintf "let %s __args__ __script_args__ =\n %s" fname body in
+    Printf.sprintf "let %s __args__ %s =\n %s" fname scriptargs body in
   (* parameter list *)
   let build_parameter_list body =
     let ctr = ref 0 in
@@ -228,9 +229,19 @@ let prepare_rule (name, metavars, script_vars, code) =
   (* add to hash table *)
   let hash_add body =
     Printf.sprintf
-      "%s\nlet _ = Hashtbl.add Coccilib.fcts \"%s\" %s\n" body name fname in
+      "%s\nlet _ = Hashtbl.add Coccilib.%s \"%s\" %s\n" body fcts name fname in
   hash_add (function_header (build_parameter_list code))
 
+let prepare_rule info =
+  prepare_generic_rule info " __script_args__" "fcts"
+
+let prepare_constraint (name, params, body) =
+  let params = (* like params of a normal script rule *)
+    List.map
+      (function (((r,n) as nm),mv) -> ((Some n,None), nm, mv, Ast.NoMVInit))
+      params in
+  prepare_generic_rule (name, params, [], body) "" "bool_fcts"
+
 let prepare coccifile code =
   let (init_mvs,sub_final_mvs,all_final_mvs) =
     let (init,final) =
@@ -272,7 +283,26 @@ let prepare coccifile code =
 	  | _ -> prev)
       [] code in
   let other_rules = List.rev other_rules in
-  if init_rules = [] && other_rules = []
+  let add_constraint_rules prev (kind, name, script_name, lang, params, body) =
+    if lang = "ocaml" then
+      let ((r,nm) as self) = Ast.unwrap_mcode name in
+      let self = (self, kind) in
+      (script_name, self::params, body) :: prev
+    else
+      prev in
+  let constraint_rules =
+    List.fold_left
+      (function prev ->
+	function
+	    Ast.CocciRule(_,_,code,_,_) ->
+	      List.fold_left
+		(fun accu toplevel ->
+		  List.fold_left add_constraint_rules prev
+		    (Parse_cocci.enumerate_constraint_scripts toplevel))
+		prev code
+	  | _ -> prev)
+      [] code in
+  if init_rules = [] && other_rules = [] && constraint_rules = []
   then None
   else
     begin
@@ -304,7 +334,9 @@ let prepare coccifile code =
       then Printf.fprintf o "%s" (String.concat "\n\n" init_rules));
       (* Semantic patch rules *)
       let rule_code = List.map prepare_rule other_rules in
-      Printf.fprintf o "%s" (String.concat "\n\n" rule_code);
+      List.iter (function rc -> Printf.fprintf o "%s\n\n" rc) rule_code;
+      let constraint_code = List.map prepare_constraint constraint_rules in
+      List.iter (function rc -> Printf.fprintf o "%s\n\n" rc) constraint_code;
       (* finalizer *)
       (if generate_final
       then
@@ -413,8 +445,7 @@ let filter_dep existing_deps (accld, accinc) dep =
     | "Tk"       -> ("tk"::accld, accinc)
 
     | _ ->
-	let l = Char.lowercase (String.get dep 0)in
-	  Bytes.set dep 0 l;
+        let dep = String.uncapitalize dep in
 	  (accld, dep::accinc)
 
 let get_dir p =
