@@ -5,7 +5,6 @@
  *)
 
 module Ast = Ast_cocci
-module TC = Type_cocci
 
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
@@ -56,7 +55,7 @@ and 'a wrap =
       info : info;
       index : int ref;
       mcodekind : mcodekind ref;
-      exp_ty : TC.typeC option ref; (* only for expressions *)
+      exp_ty : typeC option ref; (* only for expressions *)
       bef_aft : dots_bef_aft; (* only for statements *)
       true_if_arg : bool; (* true if "arg_exp", only for exprs *)
       true_if_test : bool; (* true if "test position", only for exprs *)
@@ -86,9 +85,9 @@ and 'a dots = 'a list wrap
 
 and base_ident =
     Id            of string mcode
-  | MetaId        of Ast.meta_name mcode * Ast.idconstraint * Ast.seed * pure
-  | MetaFunc      of Ast.meta_name mcode * Ast.idconstraint * pure
-  | MetaLocalFunc of Ast.meta_name mcode * Ast.idconstraint * pure
+  | MetaId        of Ast.meta_name mcode * Ast.general_constraint * Ast.seed * pure
+  | MetaFunc      of Ast.meta_name mcode * Ast.general_constraint * pure
+  | MetaLocalFunc of Ast.meta_name mcode * Ast.general_constraint * pure
   | AsIdent       of ident * ident (* as ident, always metavar *)
   | DisjId        of string mcode * ident list *
                      string mcode list (* the |s *) * string mcode
@@ -132,7 +131,7 @@ and base_expression =
 	              initialiser
   | MetaErr        of Ast.meta_name mcode * constraints * pure
   | MetaExpr       of Ast.meta_name mcode * constraints *
-	              TC.typeC list option * Ast.form * pure
+	              typeC list option * Ast.form * pure
   | MetaExprList   of Ast.meta_name mcode (* only in arg lists *) *
 	              listlen * pure
   | AsExpr         of expression * expression (* as expr, always metavar *)
@@ -142,6 +141,8 @@ and base_expression =
 	              string mcode list (* the |s *) * string mcode
   | ConjExpr       of string mcode * expression list *
 	              string mcode list (* the &s *) * string mcode
+  (* not clear why NestExpr allows dots on the expression, nor why it has a
+     whencode field *)
   | NestExpr       of string mcode * expression dots * string mcode *
 	              (string mcode * string mcode * expression) option
 	              (* whencode *) * Ast.multi
@@ -153,7 +154,7 @@ and expression = base_expression wrap
 
 and constraints =
     NoConstraint
-  | NotIdCstrt     of Ast.reconstraint
+  | NotIdCstrt     of Ast.general_constraint
   | NotExpCstrt    of expression list
   | SubExpCstrt    of Ast.meta_name list
 
@@ -172,7 +173,7 @@ and string_fragment = base_string_fragment wrap
 
 and base_string_format =
     ConstantFormat of string mcode
-  | MetaFormat of Ast.meta_name mcode * Ast.idconstraint
+  | MetaFormat of Ast.meta_name mcode * Ast.general_constraint
 
 and string_format = base_string_format wrap
 
@@ -181,22 +182,15 @@ and string_format = base_string_format wrap
 and base_assignOp =
     SimpleAssign of simpleAssignOp mcode
   | OpAssign of Ast_cocci.arithOp mcode
-  | MetaAssign of Ast_cocci.meta_name mcode * assignOpconstraint * pure
+  | MetaAssign of Ast_cocci.meta_name mcode * Ast.general_constraint * pure
 and simpleAssignOp = string
 and assignOp = base_assignOp wrap
 
 and base_binaryOp =
     Arith of Ast_cocci.arithOp mcode
   | Logical of Ast_cocci.logicalOp mcode
-  | MetaBinary of Ast_cocci.meta_name mcode * binaryOpconstraint * pure
+  | MetaBinary of Ast_cocci.meta_name mcode * Ast.general_constraint * pure
 and binaryOp = base_binaryOp wrap
-and assignOpconstraint =
-    AssignOpNoConstraint
-  | AssignOpInSet of assignOp list
-
-and binaryOpconstraint =
-    BinaryOpNoConstraint
-  | BinaryOpInSet of binaryOp list
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -238,10 +232,10 @@ and typeC = base_typeC wrap
 and base_declaration =
     MetaDecl of Ast.meta_name mcode * pure (* variables *)
     (* the following are kept separate from MetaDecls because ultimately
-       they don't match the same thin at all.  Consider whether there
+       they don't match the same thing at all.  Consider whether there
        should be a separate type for fields, as in the C AST *)
   | MetaField of Ast.meta_name mcode * pure (* structure fields *)
-  | MetaFieldList of Ast.meta_name mcode * listlen * pure (* structure fields *)
+  | MetaFieldList of Ast.meta_name mcode * listlen * pure (*structure fields*)
   | AsDecl        of declaration * declaration
   | Init of Ast.storage mcode option * typeC * ident * string mcode (*=*) *
 	initialiser * string mcode (*;*)
@@ -324,6 +318,7 @@ and parameter_list = parameterTypeDef dots
 
 and base_define_param =
     DParam        of ident
+  | MetaDParamList of Ast.meta_name mcode * listlen * pure
   | DPComma       of string mcode
   | DPdots        of string mcode (* ... *)
   | OptDParam     of define_param
@@ -461,7 +456,8 @@ and exec_code = base_exec_code wrap
 (* Positions *)
 
 and meta_pos =
-    MetaPos of Ast.meta_name mcode * Ast.meta_name list * Ast.meta_collect
+    MetaPos of Ast.meta_name mcode * Ast.general_constraint *
+	Ast.meta_collect
 
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
@@ -482,7 +478,8 @@ and parsed_rule =
       (rule (*minus*) * Ast.metavar list (*minus metavars*) *
 	(string list (*isos*) * string list (*drop_isos*) *
          Ast.dependency (*dependencies*) * string (*rulename*) * Ast.exists)) *
-      (rule (*plus*) * Ast.metavar list (*plus metavars*)) * Ast.ruletype
+      (rule (*plus*) * Ast.metavar list (*plus metavars*)) *
+	Ast.metavar list (* inhertited metavars *) * Ast.ruletype
   | ScriptRule of string (* name *) * string * Ast.dependency *
 	(Ast.script_meta_name * Ast.meta_name * Ast.metavar * Ast.mvinit)
 	  list *
@@ -506,6 +503,7 @@ and dependency =
   | NeverDep of string (* rule never applies for any binding *)
   | AndDep of dependency * dependency
   | OrDep of dependency * dependency
+  | FileIn of string
   | NoDep | FailDep
 
 (* --------------------------------------------------------------------- *)
@@ -517,6 +515,7 @@ and anything =
   | DotsStmtTag of statement dots
   | DotsDeclTag of declaration dots
   | DotsCaseTag of case_line dots
+  | DotsDefParamTag of define_param dots
   | IdentTag of ident
   | ExprTag of expression
   | AssignOpTag of assignOp
@@ -546,6 +545,7 @@ let dotsInit x = DotsInitTag x
 let dotsStmt x = DotsStmtTag x
 let dotsDecl x = DotsDeclTag x
 let dotsCase x = DotsCaseTag x
+let dotsDefParam x = DotsDefParamTag x
 let ident x = IdentTag x
 let expr x = ExprTag x
 let assignOp x = AssignOpTag x
@@ -623,6 +623,7 @@ let set_index x i   = x.index := i
 let get_mcodekind x = !(x.mcodekind)
 let get_mcode_mcodekind (_,_,_,mcodekind,_,_) = mcodekind
 let get_mcode_line (_,_,info,_,_,_) = info.pos_info.line_start
+let get_mcode_logline (_,_,info,_,_,_) = info.pos_info.logical_start
 let get_mcodekind_ref x = x.mcodekind
 let set_mcodekind x mk  = x.mcodekind := mk
 let set_type x t        = x.exp_ty := t
@@ -640,7 +641,7 @@ let get_iso x           = x.iso_info
 let set_iso x i = if !Flag.track_iso_usage then {x with iso_info = i} else x
 let set_mcode_data data (_,ar,info,mc,pos,adj) = (data,ar,info,mc,pos,adj)
 let get_rule_name = function
-  | CocciRule ((_,_,(_,_,_,nm,_)),_,_) | InitialScriptRule (nm,_,_,_,_)
+  | CocciRule ((_,_,(_,_,_,nm,_)),_,_,_) | InitialScriptRule (nm,_,_,_,_)
   | FinalScriptRule (nm,_,_,_,_) | ScriptRule (nm,_,_,_,_,_) -> nm
 
 (* --------------------------------------------------------------------- *)
@@ -649,41 +650,73 @@ let rec meta_pos_name = function
     HiddenVarTag(vars) ->
 	(* totally fake, just drop the rest, only for isos *)
       meta_pos_name (List.hd vars)
-  | MetaPosTag(MetaPos(name,constraints,_)) -> name
+  | MetaPosTag(MetaPos(name,_constraints,_)) -> name
   | IdentTag(i) ->
       (match unwrap i with
-	MetaId(name,constraints,seed,pure) -> name
+	MetaId(name,_constraints,_seed,_pure) -> name
       | _ -> failwith "bad metavariable")
   | ExprTag(e) ->
       (match unwrap e with
-	MetaExpr(name,constraints,ty,form,pure) -> name
-      | MetaExprList(name,len,pure) -> name
+	MetaExpr(name,_constraints,_ty,_form,_pure) -> name
+      | MetaExprList(name,_len,_pure) -> name
       | _ -> failwith "bad metavariable")
   | TypeCTag(t) ->
       (match unwrap t with
-	MetaType(name,pure) -> name
+	MetaType(name,_pure) -> name
       | _ -> failwith "bad metavariable")
   | DeclTag(d) ->
       (match unwrap d with
-	MetaDecl(name,pure) -> name
+	MetaDecl(name,_pure) -> name
       | _ -> failwith "bad metavariable")
   | InitTag(i) ->
       (match unwrap i with
-	MetaInit(name,pure) -> name
+	MetaInit(name,_pure) -> name
       | _ -> failwith "bad metavariable")
   | StmtTag(s) ->
       (match unwrap s with
-	MetaStmt(name,pure) -> name
+	MetaStmt(name,_pure) -> name
       | _ -> failwith "bad metavariable")
   | _ -> failwith "bad metavariable"
 
-let rec meta_pos_constraint_names = function
+let rec meta_names_of_ident ident =
+  match unwrap ident with
+    Id _ -> []
+  | MetaId (tyname, _, _, _)
+  | MetaFunc (tyname, _, _)
+  | MetaLocalFunc (tyname, _, _) -> [unwrap_mcode tyname]
+  | AsIdent (ident0, ident1) ->
+      meta_names_of_ident ident0 @ meta_names_of_ident ident1
+  | DisjId (_, l, _, _) -> List.flatten (List.map meta_names_of_ident l)
+  | OptIdent ident' -> meta_names_of_ident ident'
+
+let meta_names_of_expression expression =
+  match unwrap expression with
+    Ident ident -> meta_names_of_ident ident
+  | _ -> []
+
+let rec meta_names_of_typeC ty =
+  match unwrap ty with
+    ConstVol (_, ty)
+  | Signed (_, Some ty)
+  | Pointer (ty, _)
+  | FunctionPointer (ty, _, _, _, _, _, _)
+  | Array (ty, _, _, _) -> meta_names_of_typeC ty
+  | EnumName (_, Some ident)
+  | StructUnionName(_, Some ident) -> meta_names_of_ident ident
+  | MetaType (tyname, _) -> [unwrap_mcode tyname]
+  | Decimal (_, _, e1, _, e2, _) ->
+      let mn1 = meta_names_of_expression e1 in
+      let mn2 = Common.default [] meta_names_of_expression e2 in
+      mn1 @ mn2
+  | _ty -> []
+
+let meta_pos_constraint_names = function
     ExprTag(e) ->
       (match unwrap e with
-	MetaExpr(name,constraints,ty,form,pure) ->
+	MetaExpr(_name,_constraints,ty,_form,_pure) ->
 	  (match ty with
 	    Some tylist ->
-	      List.fold_left (fun prev cur -> TC.meta_names cur @ prev)
+              List.fold_left (fun prev cur -> meta_names_of_typeC cur @ prev)
 		[] tylist
 	  | None -> [])
       |	_ -> [])
@@ -695,197 +728,15 @@ let rec meta_pos_constraint_names = function
 let index_counter = ref 0
 let fresh_index _ = let cur = !index_counter in index_counter := cur + 1; cur
 
-(* --------------------------------------------------------------------- *)
-
-let rec ast0_type_to_type inmeta ty =
-  match unwrap ty with
-    ConstVol(cv,ty) -> TC.ConstVol(const_vol cv,ast0_type_to_type inmeta ty)
-  | BaseType(bty,strings) ->
-      TC.BaseType(baseType bty)
-  | Signed(sgn,None) ->
-      TC.SignedT(sign sgn,None)
-  | Signed(sgn,Some ty) ->
-      let bty = ast0_type_to_type inmeta ty in
-      TC.SignedT(sign sgn,Some bty)
-  | Pointer(ty,_) -> TC.Pointer(ast0_type_to_type inmeta ty)
-  | FunctionPointer(ty,_,_,_,_,params,_) ->
-      TC.FunctionPointer(ast0_type_to_type inmeta ty)
-  | Array(ety,_,_,_) -> TC.Array(ast0_type_to_type inmeta ety)
-  | Decimal(_, _, e1, _, e2, _) ->
-      let e2tc e =
-	match unwrap e with
-	  Constant(c) ->
-	    (match unwrap_mcode c with
-	      Ast.Int n -> TC.Num n
-	    | _ -> failwith "not possible")
-	| Ident(id) ->
-	    (match unwrap id with
-	      Id n -> TC.Name (unwrap_mcode n)
-	    | _ -> failwith "not possible")
-	| MetaExpr(name,NoConstraint,None,Ast.CONST,_) ->
-	    TC.MV(unwrap_mcode name,TC.Unitary,false)
-	| _ -> failwith "unexpected argument to decimal" in
-      let e2 =
-	match e2 with
-	  None -> TC.Num "0"
-	| Some e2 -> e2tc e2 in
-      TC.Decimal(e2tc e1,e2)
-  | EnumName(su,Some tag) ->
-      (match unwrap tag with
-	Id(tag) ->
-	  TC.EnumName(TC.Name(unwrap_mcode tag))
-      | MetaId(tag,Ast.IdNoConstraint,_,_) when inmeta ->
-	  TC.EnumName(TC.MV(unwrap_mcode tag,TC.Unitary,false))
-      | MetaId(tag,_,_,_) when inmeta ->
-	  (* would have to duplicate the type in type_cocci.ml?
-	     perhaps polymorphism would help? *)
-	  failwith "constraints not supported on enum type name"
-      | _ ->
-	  (* can't arise for metavariables and doesn't matter for type
-	     checking *)
-	  TC.EnumName(TC.NoName))
-  | EnumName(su,None) -> TC.EnumName TC.NoName
-  | EnumDef(ty,_,_,_) -> ast0_type_to_type inmeta ty
-  | StructUnionName(su,Some tag) ->
-      (match unwrap tag with
-	Id(tag) ->
-	  TC.StructUnionName(structUnion su,TC.Name(unwrap_mcode tag))
-      | MetaId(tag,Ast.IdNoConstraint,_,_) when inmeta ->
-	  TC.StructUnionName(structUnion su,
-			     TC.MV(unwrap_mcode tag,TC.Unitary,false))
-      | MetaId(tag,_,_,_) when inmeta ->
-	  (* would have to duplicate the type in type_cocci.ml?
-	     perhaps polymorphism would help? *)
-	  failwith "constraints not supported on struct type name"
-      | _ ->
-	  (* can't arise for metavariables and doesn't matter for type
-	     checking *)
-	  TC.StructUnionName(structUnion su,TC.NoName))
-  | StructUnionName(su,None) -> TC.StructUnionName(structUnion su,TC.NoName)
-  | StructUnionDef(ty,_,_,_) -> ast0_type_to_type inmeta ty
-  | TypeName(name) -> TC.TypeName(unwrap_mcode name)
-  | MetaType(name,_) ->
-      TC.MetaType(unwrap_mcode name,TC.Unitary,false)
-  | AsType(ty,asty) -> failwith "not created yet"
-  | DisjType(_,types,_,_) ->
-      Common.pr2_once
-	"disjtype not supported in smpl type inference, assuming unknown";
-      TC.Unknown
-  | OptType(ty) -> ast0_type_to_type inmeta ty
-
-and baseType = function
-    Ast.VoidType -> TC.VoidType
-  | Ast.CharType -> TC.CharType
-  | Ast.ShortType -> TC.ShortType
-  | Ast.ShortIntType -> TC.ShortIntType
-  | Ast.IntType -> TC.IntType
-  | Ast.DoubleType -> TC.DoubleType
-  | Ast.LongDoubleType -> TC.LongDoubleType
-  | Ast.FloatType -> TC.FloatType
-  | Ast.LongType -> TC.LongType
-  | Ast.LongIntType -> TC.LongIntType
-  | Ast.LongLongType -> TC.LongLongType
-  | Ast.LongLongIntType -> TC.LongLongIntType
-  | Ast.SizeType -> TC.SizeType
-  | Ast.SSizeType -> TC.SSizeType
-  | Ast.PtrDiffType -> TC.PtrDiffType
-
-and structUnion t =
-  match unwrap_mcode t with
-    Ast.Struct -> TC.Struct
-  | Ast.Union -> TC.Union
-
-and sign t =
-  match unwrap_mcode t with
-    Ast.Signed -> TC.Signed
-  | Ast.Unsigned -> TC.Unsigned
-
-and const_vol t =
-  match unwrap_mcode t with
-    Ast.Const -> TC.Const
-  | Ast.Volatile -> TC.Volatile
-
 (* --------------------------------------------------------------------- *)
 (* this function is a rather minimal attempt.  the problem is that information
 has been lost.  but since it is only used for metavariable types in the isos,
 perhaps it doesn't matter *)
-and make_mcode x = (x,NONE,default_info(),context_befaft(),ref [],-1)
+let make_mcode x = (x,NONE,default_info(),context_befaft(),ref [],-1)
 let make_mcode_info x info = (x,NONE,info,context_befaft(),ref [],-1)
 and make_minus_mcode x =
   (x,NONE,default_info(),minus_befaft(),ref [],-1)
 
-exception TyConv
-
-let rec reverse_type ty =
-  match ty with
-    TC.ConstVol(cv,ty) ->
-      ConstVol(reverse_const_vol cv,context_wrap(reverse_type ty))
-  | TC.BaseType(bty) ->
-      BaseType(reverse_baseType bty,[(* not used *)])
-  | TC.SignedT(sgn,None) -> Signed(reverse_sign sgn,None)
-  | TC.SignedT(sgn,Some bty) ->
-      Signed(reverse_sign sgn,Some (context_wrap(reverse_type ty)))
-  | TC.Pointer(ty) ->
-      Pointer(context_wrap(reverse_type ty),make_mcode "*")
-  | TC.EnumName(TC.MV(name,_,_)) ->
-      EnumName
-	(make_mcode "enum",
-	 Some (context_wrap(MetaId(make_mcode name,Ast.IdNoConstraint,Ast.NoVal,
-				   Impure))))
-  | TC.EnumName(TC.Name tag) ->
-      EnumName(make_mcode "enum",Some(context_wrap(Id(make_mcode tag))))
-  | TC.StructUnionName(su,TC.MV(name,_,_)) ->
-      (* not right?... *)
-      StructUnionName
-	(reverse_structUnion su,
-	 Some(context_wrap(MetaId(make_mcode name,Ast.IdNoConstraint,Ast.NoVal,
-				  Impure(*not really right*)))))
-  |  TC.StructUnionName(su,TC.Name tag) ->
-      StructUnionName
-	(reverse_structUnion su,
-	 Some (context_wrap(Id(make_mcode tag))))
-  | TC.TypeName(name) -> TypeName(make_mcode name)
-  | TC.MetaType(name,_,_) ->
-      MetaType(make_mcode name,Impure(*not really right*))
-  | _ -> raise TyConv
-
-and reverse_baseType = function
-    TC.VoidType -> Ast.VoidType
-  | TC.CharType -> Ast.CharType
-  | TC.BoolType -> Ast.IntType
-  | TC.ShortType -> Ast.ShortType
-  | TC.ShortIntType -> Ast.ShortIntType
-  | TC.IntType -> Ast.IntType
-  | TC.DoubleType -> Ast.DoubleType
-  | TC.LongDoubleType -> Ast.LongDoubleType
-  | TC.FloatType -> Ast.FloatType
-  | TC.LongType -> Ast.LongType
-  | TC.LongIntType -> Ast.LongIntType
-  | TC.LongLongType -> Ast.LongLongType
-  | TC.LongLongIntType -> Ast.LongLongIntType
-  | TC.SizeType -> Ast.SizeType
-  | TC.SSizeType -> Ast.SSizeType
-  | TC.PtrDiffType -> Ast.PtrDiffType
-
-
-and reverse_structUnion t =
-  make_mcode
-    (match t with
-      TC.Struct -> Ast.Struct
-    | TC.Union -> Ast.Union)
-
-and reverse_sign t =
-  make_mcode
-    (match t with
-      TC.Signed -> Ast.Signed
-    | TC.Unsigned -> Ast.Unsigned)
-
-and reverse_const_vol t =
-  make_mcode
-    (match t with
-      TC.Const -> Ast.Const
-    | TC.Volatile -> Ast.Volatile)
-
 (* --------------------------------------------------------------------- *)
 
 let lub_pure x y =
@@ -912,3 +763,8 @@ let string_of_assignOp op = match (unwrap op) with
     let s = string_of_binaryOp op'' in
     s ^ "="
   | MetaAssign _ -> "MetaAssign"
+
+let is_unknown_type ty =
+  match unwrap ty with
+    BaseType (Ast.Unknown, _) -> true
+  | _ -> false
