@@ -47,7 +47,7 @@ and 'a wrap =
       info : info;
       index : int ref;
       mcodekind : mcodekind ref;
-      exp_ty : Type_cocci.typeC option ref; (* only for expressions *)
+      exp_ty : typeC option ref; (* only for expressions *)
       bef_aft : dots_bef_aft; (* only for statements *)
       true_if_arg : bool; (* true if "arg_exp", only for exprs *)
       true_if_test : bool; (* true if "test position", only for exprs *)
@@ -75,10 +75,10 @@ and 'a dots = 'a list wrap
 
 and base_ident =
     Id            of string mcode
-  | MetaId        of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint *
+  | MetaId        of Ast_cocci.meta_name mcode * Ast_cocci.general_constraint *
 	Ast_cocci.seed * pure
-  | MetaFunc      of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint * pure
-  | MetaLocalFunc of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint * pure
+  | MetaFunc      of Ast_cocci.meta_name mcode * Ast_cocci.general_constraint * pure
+  | MetaLocalFunc of Ast_cocci.meta_name mcode * Ast_cocci.general_constraint * pure
   | AsIdent       of ident * ident (* as ident, always metavar *)
   | DisjId        of string mcode * ident list *
                      string mcode list (* the |s *) * string mcode
@@ -122,7 +122,7 @@ and base_expression =
  	              initialiser
   | MetaErr        of Ast_cocci.meta_name mcode * constraints * pure
   | MetaExpr       of Ast_cocci.meta_name mcode * constraints *
-	              Type_cocci.typeC list option * Ast_cocci.form * pure
+	              typeC list option * Ast_cocci.form * pure
   | MetaExprList   of Ast_cocci.meta_name mcode (* only in arglists *) *
 	              listlen * pure
   | AsExpr         of expression * expression (* as expr, always metavar *)
@@ -143,7 +143,7 @@ and expression = base_expression wrap
 
 and constraints =
     NoConstraint
-  | NotIdCstrt     of Ast_cocci.reconstraint
+  | NotIdCstrt     of Ast_cocci.general_constraint
   | NotExpCstrt    of expression list
   | SubExpCstrt    of Ast_cocci.meta_name list
 
@@ -162,32 +162,26 @@ and string_fragment = base_string_fragment wrap
 
 and base_string_format =
     ConstantFormat of string mcode
-  | MetaFormat of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint
+  | MetaFormat of Ast_cocci.meta_name mcode * Ast_cocci.general_constraint
 
 and string_format = base_string_format wrap
 
 (* --------------------------------------------------------------------- *)
 (* First class operators *)
-and  base_assignOp =
+and base_assignOp =
     SimpleAssign of simpleAssignOp mcode
   | OpAssign of Ast_cocci.arithOp mcode
-  | MetaAssign of Ast_cocci.meta_name mcode * assignOpconstraint * pure
+  | MetaAssign of
+      Ast_cocci.meta_name mcode * Ast_cocci.general_constraint * pure
 and simpleAssignOp = string
 and assignOp = base_assignOp wrap
 
-and  base_binaryOp =
+and base_binaryOp =
     Arith of Ast_cocci.arithOp mcode
   | Logical of Ast_cocci.logicalOp mcode
-  | MetaBinary of Ast_cocci.meta_name mcode * binaryOpconstraint * pure
+  | MetaBinary of
+      Ast_cocci.meta_name mcode * Ast_cocci.general_constraint * pure
 and binaryOp = base_binaryOp wrap
-and assignOpconstraint =
-    AssignOpNoConstraint
-  | AssignOpInSet of assignOp list
-
-and binaryOpconstraint =
-    BinaryOpNoConstraint
-  | BinaryOpInSet of binaryOp list
-
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -310,6 +304,7 @@ and parameter_list = parameterTypeDef dots
 
 and base_define_param =
     DParam        of ident
+  | MetaDParamList of Ast_cocci.meta_name mcode * listlen * pure
   | DPComma       of string mcode
   | DPdots        of string mcode (* ... *)
   | OptDParam     of define_param
@@ -446,7 +441,7 @@ and exec_code = base_exec_code wrap
 (* Positions *)
 
 and meta_pos =
-    MetaPos of Ast_cocci.meta_name mcode * Ast_cocci.meta_name list *
+    MetaPos of Ast_cocci.meta_name mcode * Ast_cocci.general_constraint *
 	Ast_cocci.meta_collect
 
 (* --------------------------------------------------------------------- *)
@@ -468,7 +463,8 @@ and parsed_rule =
       (rule * Ast_cocci.metavar list *
 	 (string list * string list * Ast_cocci.dependency * string *
 	    Ast_cocci.exists)) *
-	(rule * Ast_cocci.metavar list) * Ast_cocci.ruletype
+	(rule * Ast_cocci.metavar list) *
+	Ast_cocci.metavar list (* inhertited metavars *) * Ast_cocci.ruletype
   | ScriptRule of string (* name *) *
       string * Ast_cocci.dependency *
 	(Ast_cocci.script_meta_name *
@@ -496,6 +492,7 @@ and dependency =
   | NeverDep of string (* rule never applies for any binding *)
   | AndDep of dependency * dependency
   | OrDep of dependency * dependency
+  | FileIn of string
   | NoDep | FailDep
 
 (* --------------------------------------------------------------------- *)
@@ -507,6 +504,7 @@ and anything =
   | DotsStmtTag of statement dots
   | DotsDeclTag of declaration dots
   | DotsCaseTag of case_line dots
+  | DotsDefParamTag of define_param dots
   | IdentTag of ident
   | ExprTag of expression
   | AssignOpTag of assignOp
@@ -536,6 +534,7 @@ val dotsParam : parameterTypeDef dots -> anything
 val dotsStmt : statement dots -> anything
 val dotsDecl : declaration dots -> anything
 val dotsCase : case_line dots -> anything
+val dotsDefParam : define_param dots -> anything
 val ident : ident -> anything
 val expr : expression -> anything
 val assignOp : assignOp -> anything
@@ -572,13 +571,14 @@ val get_index : 'a wrap -> int
 val set_index : 'a wrap -> int -> unit
 val get_line : 'a wrap -> int
 val get_mcode_line : 'a mcode -> int
+val get_mcode_logline : 'a mcode -> int
 val get_line_end : 'a wrap -> int
 val get_mcodekind : 'a wrap -> mcodekind
 val get_mcode_mcodekind : 'a mcode -> mcodekind
 val get_mcodekind_ref : 'a wrap -> mcodekind ref
 val set_mcodekind : 'a wrap -> mcodekind -> unit
-val set_type : 'a wrap -> Type_cocci.typeC option -> unit
-val get_type : 'a wrap -> Type_cocci.typeC option
+val set_type : 'a wrap -> typeC option -> unit
+val get_type : 'a wrap -> typeC option
 val set_dots_bef_aft : statement -> dots_bef_aft -> statement
 val get_dots_bef_aft : 'a wrap -> dots_bef_aft
 val set_arg_exp : expression -> expression
@@ -598,12 +598,12 @@ val make_minus_mcode : 'a -> 'a mcode
 val get_rule_name : parsed_rule -> string
 
 val meta_pos_name : anything -> Ast_cocci.meta_name mcode
+val meta_names_of_typeC : typeC -> Ast_cocci.meta_name list
+(**
+ * [meta_names_of_typeC ty] enumerates all the meta names that occur in [ty].
+ *)
 val meta_pos_constraint_names : anything -> Ast_cocci.meta_name list
 
-val ast0_type_to_type : bool -> typeC -> Type_cocci.typeC
-val reverse_type : Type_cocci.typeC -> base_typeC
-exception TyConv
-
 val lub_pure : pure -> pure -> pure
 
 (* --------------------------------------------------------------------- *)
@@ -612,3 +612,9 @@ val rule_name : string ref (* for the convenience of the parser *)
 
 val string_of_assignOp : assignOp -> string
 val string_of_binaryOp : binaryOp -> string
+
+val is_unknown_type : typeC -> bool
+(**
+ * [is_unknown_type ty] returns true iff [ty] is an unknown type
+ * (i.e., of the form [BaseType (Ast_cocci.Unknown, _)]).
+ *)
