@@ -13,6 +13,7 @@ module Ast = Ast_cocci
 module Ast0 = Ast0_cocci
 module V0 = Visitor_ast0
 module VT0 = Visitor_ast0_types
+module U = Unparse_ast0
 
 let comm_assoc_arith =
   [Ast.Plus; Ast.Mul; Ast.And; Ast.Or]
@@ -54,16 +55,17 @@ let process_binops rule_name =
 	    (* keep dots to record required modif *)
 	    then Ast0.rewrap e (Ast0.Nested(left,op,right))
 	    else
-	      (Printf.printf
-		 "%s: position variables or mixed modifs interfere with comm_assoc iso" rule_name;
-	       Unparse_ast0.expression e1;
-	       Format.print_newline();
+	      (Common.pr2
+		 (Printf.sprintf
+		    "%s: position variables or mixed modifs interfere with comm_assoc iso\n%s"
+		    rule_name
+		    (U.unparse_x_to_string U.expression e1));
 	       e)
 	| Ast0.Edots(d,_) ->
-	    (Printf.printf
-	       "%s: whencode interferes with comm_assoc iso" rule_name;
-	     Unparse_ast0.expression e1;
-	     Format.print_newline();
+	    (Common.pr2
+	       (Printf.sprintf
+		  "%s: whencode interferes with comm_assoc iso\n%s" rule_name
+		  (U.unparse_x_to_string U.expression e1));
 	     e)
 	| _ -> e)
     | _ -> e in
