@@ -28,12 +28,12 @@ let rec disjmult f = function
       let rest = disjmult f xs in
       disjmult2 cur rest (function cur -> function rest -> cur :: rest)
 
-let rec disjmult_two fstart frest (start,rest) =
+let disjmult_two fstart frest (start,rest) =
   let cur = fstart start in
   let rest = disjmult frest rest in
   disjmult2 cur rest (function cur -> function rest -> (cur,rest))
 
-let rec disjtwoelems fstart frest (start,rest) =
+let disjtwoelems fstart frest (start,rest) =
   let cur = fstart start in
   let rest = frest rest in
   disjmult2 cur rest (function cur -> function rest -> (cur,rest))
@@ -260,7 +260,9 @@ and disjini i =
       List.map (function ini -> Ast.rewrap i (Ast.OptIni(ini))) ini
 
 and designator = function
-    Ast.DesignatorField(dot,id) -> [Ast.DesignatorField(dot,id)]
+    Ast.DesignatorField(dot,id) ->
+      let id = disjident id in
+      List.map (function id -> Ast.DesignatorField(dot,id)) id
   | Ast.DesignatorIndex(lb,exp,rb) ->
       let exp = disjexp exp in
       List.map (function exp -> Ast.DesignatorIndex(lb,exp,rb)) exp
@@ -429,7 +431,7 @@ let disj_all =
     mcode mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing
+    donothing donothing donothing
     donothing disj_rule_elem donothing donothing donothing donothing
 
 (* ----------------------------------------------------------------------- *)
@@ -446,7 +448,7 @@ let collect_all_isos =
     mcode mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing
+    donothing donothing donothing
     doanything donothing donothing donothing donothing doanything
 
 let collect_iso_info =
@@ -463,7 +465,7 @@ let collect_iso_info =
     mcode mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing
-    donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing
     donothing donothing donothing donothing rule_elem donothing donothing
     donothing donothing
 
