@@ -64,7 +64,7 @@ let drop_positions =
     V0.flat_rebuilder
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode
-    donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing in
   res.VT0.rebuilder_rec_statement
@@ -168,7 +168,7 @@ and strip =
   V0.flat_rebuilder
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode
-    donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing donothing donothing
     ident donothing donothing donothing typeC donothing param
     donothing donothing donothing donothing donothing donothing
 
@@ -181,7 +181,7 @@ and changed_proto = function
 (* --------------------------------------------------------------------- *)
 (* make rules *)
 
-let rec collect_ident_strings id =
+let collect_ident_strings id =
   let bind x y = x @ y in
   let option_default = [] in
   let donothing r k e = k e in
@@ -191,7 +191,7 @@ let rec collect_ident_strings id =
     V0.flat_combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
       mcode mcode
-      donothing donothing donothing donothing donothing donothing
+      donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing in
       v.VT0.combiner_rec_ident id
