@@ -75,6 +75,7 @@ it *)
       (donothing Ast0.dotsExpr) (donothing Ast0.dotsInit)
       (donothing Ast0.dotsParam) (donothing Ast0.dotsStmt)
       (donothing Ast0.dotsDecl) (donothing Ast0.dotsCase)
+      (donothing Ast0.dotsDefParam)
       (donothing Ast0.ident) expression  (donothing Ast0.assignOp)
       (donothing Ast0.binaryOp)
       (donothing Ast0.typeC) initialiser
@@ -111,6 +112,7 @@ let create_root_token_table minus =
 	  | Ast0.DotsStmtTag(d) -> Ast0.get_index d
 	  | Ast0.DotsDeclTag(d) -> Ast0.get_index d
 	  | Ast0.DotsCaseTag(d) -> Ast0.get_index d
+	  | Ast0.DotsDefParamTag(d) -> Ast0.get_index d
 	  | Ast0.IdentTag(d) -> Ast0.get_index d
 	  | Ast0.ExprTag(d) -> Ast0.get_index d
 	  | Ast0.AssignOpTag d -> Ast0.get_index d
@@ -169,7 +171,7 @@ let collect_minus_join_points root =
 (* don't want to attach to the outside of DOTS, because metavariables can't
 bind to that; not good for isomorphisms *)
 
-  let dots f k d checklast =
+  let dots f k d =
     let multibind l =
       let rec loop = function
 	  [] -> option_default
@@ -178,39 +180,41 @@ bind to that; not good for isomorphisms *)
       loop l in
 
     let l = Ast0.unwrap d in
-    match checklast with
-      None -> multibind (List.map f l)
-    | Some checklast ->
-	match List.rev l with
-	  last::others ->
-	    multibind ((List.map f (List.rev others)) @ [checklast last])
-	| [] -> multibind [] in
+    multibind (List.map f l) in
 
-  (* have to unfavor a trailing comme because it might not match anything *)
-  let edots r k d = dots r.VT0.combiner_rec_expression k d
-      (Some
-	 (function e ->
-	   match Ast0.unwrap e with
-	     Ast0.EComma(comma) -> unfavored_mcode comma
-	   | _ -> r.VT0.combiner_rec_expression e))
-  in
-  let idots r k d = dots r.VT0.combiner_rec_initialiser k d
-      (Some
-	 (function i ->
-	   match Ast0.unwrap i with
-	     Ast0.IComma(comma) -> unfavored_mcode comma
-	   | _ -> r.VT0.combiner_rec_initialiser i)) in
+  (* just unfavor all commas *)
+  let edots r k d =
+    dots
+      (function e ->
+	match Ast0.unwrap e with
+	  Ast0.EComma(comma) -> unfavored_mcode comma
+	| _ -> r.VT0.combiner_rec_expression e)
+      k d in
+  let idots r k d =
+    dots
+      (function i ->
+	match Ast0.unwrap i with
+	  Ast0.IComma(comma) -> unfavored_mcode comma
+	| _ -> r.VT0.combiner_rec_initialiser i)
+      k d in
+  let pdots r k d =
+    dots
+      (function p ->
+	match Ast0.unwrap p with
+	  Ast0.PComma(comma) -> unfavored_mcode comma
+	| _ -> r.VT0.combiner_rec_parameter p)
+  k d in
+  let dpdots r k d =
+    dots
+      (function p ->
+	match Ast0.unwrap p with
+	  Ast0.DPComma(comma) -> unfavored_mcode comma
+	| _ -> r.VT0.combiner_rec_define_param p)
+  k d in
 
-  let pdots r k d = dots r.VT0.combiner_rec_parameter k d
-      (Some
-	 (function p ->
-	   match Ast0.unwrap p with
-	     Ast0.PComma(comma) -> unfavored_mcode comma
-	   | _ -> r.VT0.combiner_rec_parameter p)) in
-
-  let sdots r k d = dots r.VT0.combiner_rec_statement k d None in
-  let ddots r k d = dots r.VT0.combiner_rec_declaration k d None in
-  let cdots r k d = dots r.VT0.combiner_rec_case_line k d None in
+  let sdots r k d = dots r.VT0.combiner_rec_statement k d in
+  let ddots r k d = dots r.VT0.combiner_rec_declaration k d in
+  let cdots r k d = dots r.VT0.combiner_rec_case_line k d in
 
   (* a case for everything that has a Opt *)
 
@@ -329,7 +333,7 @@ bind to that; not good for isomorphisms *)
   V0.flat_combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode
-    edots idots pdots sdots ddots cdots
+    edots idots pdots sdots ddots cdots dpdots
     ident expression do_nothing do_nothing
     typeC initialiser param decl statement forinfo
     case_line do_nothing do_top
@@ -358,6 +362,9 @@ let call_collect_minus context_nodes :
       | Ast0.DotsCaseTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_minus_join_points e).VT0.combiner_rec_case_line_dots e)
+      | Ast0.DotsDefParamTag(e) ->
+	  (Ast0.get_index e,
+	   (collect_minus_join_points e).VT0.combiner_rec_define_param_dots e)
       | Ast0.IdentTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_minus_join_points e).VT0.combiner_rec_ident e)
@@ -485,6 +492,7 @@ let mk_paramdots x = Ast.ParamDotsTag (Ast0toast.parameter_list x)
 let mk_stmtdots x  = Ast.StmtDotsTag (Ast0toast.statement_dots x)
 let mk_decldots x  = Ast.AnnDeclDotsTag (Ast0toast.declaration_dots x)
 let mk_casedots x  = failwith "+ case lines not supported"
+let mk_defpardots x= Ast.DefParDotsTag (Ast0toast.define_param_dots x)
 let mk_typeC x     = Ast.FullTypeTag (Ast0toast.typeC false x)
 let mk_init x      = Ast.InitTag (Ast0toast.initialiser x)
 let mk_param x     = Ast.ParamTag (Ast0toast.parameterTypeDef x)
@@ -589,7 +597,7 @@ let collect_plus_nodes root =
     (mcode mk_storage) (mcode mk_inc_file)
     (do_nothing mk_exprdots) initdots
     (do_nothing mk_paramdots) stmt_dots (do_nothing mk_decldots)
-    (do_nothing mk_casedots)
+    (do_nothing mk_casedots) (do_nothing mk_defpardots)
     (do_nothing mk_ident) (do_nothing mk_expression)
     (do_nothing mk_assignOp) (do_nothing mk_binaryOp)
     (do_nothing mk_typeC) (do_nothing mk_init) (do_nothing mk_param)
@@ -620,6 +628,9 @@ let call_collect_plus context_nodes :
       | Ast0.DotsCaseTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_case_line_dots e)
+      | Ast0.DotsDefParamTag(e) ->
+	  (Ast0.get_index e,
+	   (collect_plus_nodes e).VT0.combiner_rec_define_param_dots e)
       | Ast0.IdentTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_ident e)
@@ -688,7 +699,7 @@ let redo info start finish =
       Ast0.logical_end = finish} in
   {info with Ast0.pos_info = new_pos_info}
 
-let rec find_neighbors (index,l) :
+let find_neighbors (index,l) :
     int * (Ast0.info * Ast.count * (Ast.anything list list)) list =
   let rec loop = function
       [] -> []
@@ -1134,7 +1145,7 @@ let reevaluate_contextness =
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
       mcode mcode
       donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing donothing donothing
+      donothing donothing donothing donothing donothing
       donothing donothing donothing stmt donothing donothing donothing
       donothing in
   res.VT0.combiner_rec_top_level
