@@ -11,10 +11,10 @@ let print_plus_flag = ref true
 let print_minus_flag = ref true
 let print_newlines_disj = ref true
 
-let start_block str =
+let start_block _str =
   force_newline(); print_string "  "; open_box 0
 
-let end_block str =
+let end_block _str =
   close_box(); force_newline ()
 
 let print_string_box s = print_string s; open_box 0
@@ -70,7 +70,7 @@ let print_string_befaft fn x info =
   fn x;
   List.iter (function (s,_,_) -> force_newline(); print s) info.Ast.straft
 
-let print_meta (r,x) = print_string x
+let print_meta (_r,x) = print_string x
 
 let print_pos l =
   List.iter
@@ -81,7 +81,7 @@ let print_pos l =
     l
 
 let mcode fn = function
-    (x, _, Ast.MINUS(_,_,adj,plus_stream), pos) ->
+    (x, _, Ast.MINUS(_,_,_adj,plus_stream), pos) ->
       if !print_minus_flag
       then print_string (if !Flag.sgrep_mode2 then "*" else "-");
       fn x; print_pos pos;
@@ -141,13 +141,13 @@ let print_disj_list fn l sep =
 
 (* --------------------------------------------------------------------- *)
 
-let print_type keep info = function
+let print_type _keep _info = function
     None -> () (*
 	;print_string "/* notype ";(*
            print_string "keep:"; print_unitary keep;
            print_string " inherited:"; print_bool inherited;*)
            print_string " */" *)
-  | Some ty -> () (*
+  | Some _ty -> () (*
      ;
       print_string "/* ";
       print_between (function _ -> print_string ", ") Type_cocci.typeC ty;(*
@@ -159,23 +159,32 @@ let print_type keep info = function
 (* Constraint on Identifier and Function *)
 (* FIXME: Not called at the moment *)
 
-let rec idconstraint = function
-    Ast.IdNoConstraint  -> print_string "/* No constraint */"
-  | Ast.IdPosIdSet (str,meta)     ->
+let general_constraint = function
+    Ast.CstrTrue -> print_string "/* No constraint */"
+  | Ast.CstrOr l when
+      List.for_all
+	(function Ast.CstrString _ | Ast.CstrMeta_name _ -> true | _ -> false)
+	l ->
       print_string " =";
-      List.iter (function s -> print_string (" "^s)) str;
-      List.iter (function (r,n) -> print_string " "; print_meta(r,n)) meta
-  | Ast.IdNegIdSet (str,meta)     ->
+      List.iter (function
+	  Ast.CstrString s -> Printf.printf " %s" s
+	| Ast.CstrMeta_name (r, n) -> print_string " "; print_meta (r, n)
+	| _ -> assert false) l
+  | Ast.CstrNot (Ast.CstrOr l) when
+      List.for_all
+	(function Ast.CstrString _ | Ast.CstrMeta_name _ -> true | _ -> false)
+	l ->
       print_string " !=";
-      List.iter (function s -> print_string (" "^s)) str;
-      List.iter (function (r,n) -> print_string " "; print_meta(r,n)) meta
-  | Ast.IdRegExpConstraint re -> regconstraint re
-
-and regconstraint = function
-    Ast.IdRegExp (re,_) ->
+      List.iter (function
+	  Ast.CstrString s -> Printf.printf " %s" s
+	| Ast.CstrMeta_name (r, n) -> print_string " "; print_meta (r, n)
+	| _ -> assert false) l
+  | Ast.CstrRegexp (re,_) ->
       print_string "~= \""; print_string re; print_string "\""
-  | Ast.IdNotRegExp (re,_) ->
+  | Ast.CstrNot (Ast.CstrRegexp (re,_)) ->
       print_string "~!= \""; print_string re; print_string "\""
+  | Ast.CstrScript _ -> print_string ": [script]"
+  | _ -> print_string ": ??"
 
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
@@ -191,9 +200,9 @@ let rec ident i =
   | Ast.OptIdent(id) -> print_string "?"; ident id
 
 and print_unitary = function
-    Type_cocci.Unitary -> print_string "unitary"
-  | Type_cocci.Nonunitary -> print_string "nonunitary"
-  | Type_cocci.Saved -> print_string "saved"
+    Ast.Unitary -> print_string "unitary"
+  | Ast.Nonunitary -> print_string "nonunitary"
+  | Ast.Saved -> print_string "saved"
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
@@ -302,7 +311,8 @@ and unaryOp = function
 and assignOp op =
   match Ast.unwrap op with
     Ast.SimpleAssign op -> mcode print_string op
-  | Ast.OpAssign(aop) -> mcode arithOp aop; print_string "="
+  | Ast.OpAssign(aop) ->
+      mcode (function op -> arithOp op; print_string "=") aop
   | Ast.MetaAssign(metavar,_,_,_) -> mcode print_meta metavar
 
 and simpleAssignOp op = print_string "="
@@ -430,34 +440,13 @@ and typeC ty =
   | Ast.MetaType(name,_,_) ->
       mcode print_meta name; print_string " "
 
-and baseType = function
-    Ast.VoidType -> print_string "void "
-  | Ast.CharType -> print_string "char "
-  | Ast.ShortType -> print_string "short "
-  | Ast.ShortIntType -> print_string "short int "
-  | Ast.IntType -> print_string "int "
-  | Ast.DoubleType -> print_string "double "
-  | Ast.LongDoubleType -> print_string "long double "
-  | Ast.FloatType -> print_string "float "
-  | Ast.LongType -> print_string "long "
-  | Ast.LongIntType -> print_string "long int "
-  | Ast.LongLongType -> print_string "long long "
-  | Ast.LongLongIntType -> print_string "long long int "
-  | Ast.SizeType -> print_string "size_t "
-  | Ast.SSizeType -> print_string "ssize_t "
-  | Ast.PtrDiffType -> print_string "ptrdiff_t "
+and baseType ty = print_string (Ast.string_of_baseType ty ^ " ")
 
-and structUnion = function
-    Ast.Struct -> print_string "struct "
-  | Ast.Union -> print_string "union "
+and structUnion ty = print_string (Ast.string_of_structUnion ty ^ " ")
 
-and sign = function
-    Ast.Signed -> print_string "signed "
-  | Ast.Unsigned -> print_string "unsigned "
+and sign s = print_string (Ast.string_of_sign s ^ " ")
 
-and const_vol = function
-    Ast.Const -> print_string "const"
-  | Ast.Volatile -> print_string "volatile"
+and const_vol const_vol = print_string (Ast.string_of_const_vol const_vol ^ " ")
 
 (* --------------------------------------------------------------------- *)
 (* Variable declaration *)
@@ -666,7 +655,7 @@ and rule_elem arity re =
       print_string arity; mcode print_string cont; mcode print_string sem
   | Ast.Label(l,dd) -> ident l; mcode print_string dd
   | Ast.Goto(goto,l,sem) ->
-      mcode print_string goto; ident l; mcode print_string sem
+      mcode print_string goto; print_space(); ident l; mcode print_string sem
   | Ast.Return(ret,sem) ->
       print_string arity; mcode print_string ret; mcode print_string sem
   | Ast.ReturnExpr(ret,exp,sem) ->
@@ -738,6 +727,7 @@ and print_define_parameters params =
 and print_define_param param =
   match Ast.unwrap param with
     Ast.DParam(id) -> ident id
+  | Ast.MetaDParamList(name,_,_,_) -> mcode print_meta name
   | Ast.DPComma(comma) -> mcode print_string comma
   | Ast.DPdots(dots) -> mcode print_string dots
   | Ast.OptDParam(dp) -> print_string "?"; print_define_param dp
@@ -821,14 +811,6 @@ and statement arity s =
       close_box(); force_newline()
   | Ast.OptStm(s) -> statement "?" s
 
-and print_statement_when whencode =
-  print_string "   WHEN != ";
-  open_box 0;
-  print_between (function _ -> print_string " &"; force_newline())
-    (dots force_newline (statement "")) whencode;
-  close_box()
-
-
 and whencode notfn alwaysfn = function
     Ast.WhenNot a ->
       print_string "   WHEN != "; open_box 0; notfn a; close_box()
@@ -836,15 +818,15 @@ and whencode notfn alwaysfn = function
       print_string "   WHEN = "; open_box 0; alwaysfn a; close_box()
   | Ast.WhenModifier x -> print_string "   WHEN "; print_when_modif x
   | Ast.WhenNotTrue a ->
-      print_string "   WHEN != TRUE "; open_box 0; rule_elem "" a; close_box()
+      print_string "   WHEN != true "; open_box 0; rule_elem "" a; close_box()
   | Ast.WhenNotFalse a ->
-      print_string "   WHEN != FALSE "; open_box 0; rule_elem "" a; close_box()
+      print_string "   WHEN != false "; open_box 0; rule_elem "" a; close_box()
 
 and print_when_modif = function
-  | Ast.WhenAny    -> print_string "ANY"
-  | Ast.WhenStrict -> print_string "STRICT"
-  | Ast.WhenForall -> print_string "FORALL"
-  | Ast.WhenExists -> print_string "EXISTS"
+  | Ast.WhenAny    -> print_string "any"
+  | Ast.WhenStrict -> print_string "strict"
+  | Ast.WhenForall -> print_string "forall"
+  | Ast.WhenExists -> print_string "exists"
 
 and case_line arity c =
   match Ast.unwrap c with
@@ -871,6 +853,7 @@ and inc_file = function
       print_string "<";
       print_between (function _ -> print_string "/") inc_elem elems;
       print_string ">"
+  | Ast.AnyInc -> print_string "..."
 
 and inc_elem = function
     Ast.IncPath s -> print_string s
@@ -896,11 +879,11 @@ let print_listlen rule = function
 
 let print_types = function
     None -> ()
-  | Some [ty] -> print_string (Type_cocci.type2c ty); print_string " "
+  | Some [ty] -> print_string (Ast.string_of_fullType ty); print_string " "
   | Some l ->
       print_string "{";
       print_between (fun _ -> print_string ",")
-	(fun ty -> print_string (Type_cocci.type2c ty))
+	(fun ty -> print_string (Ast.string_of_fullType ty))
 	l;
       print_string "} "
 
@@ -915,7 +898,16 @@ let print_seed rule = function
       print_string " = ";
       print_between (fun _ -> print_string " ## ") (print_seed_elem rule) ss
 
-let rec unparse_cocci_mv rule = function
+let contains_unknown ty =
+  try
+    Ast.fullType_iter { Ast.empty_transformer with
+      Ast.baseType =
+	Some (fun ty _ -> match ty with Ast.Unknown -> raise Exit | _ -> ())
+    } ty;
+    false
+  with Exit -> true
+
+let unparse_cocci_mv rule = function
     Ast.MetaMetaDecl _ -> failwith "should be removed"
   | Ast.MetaIdDecl(_,(r,n)) ->
       print_string "identifier "; print_name rule r n; print_string ";"
@@ -929,7 +921,7 @@ let rec unparse_cocci_mv rule = function
   | Ast.MetaInitListDecl(_,(r,n),len) ->
       print_string "initializer list"; print_listlen rule len;
       print_name rule r n; print_string ";"
-  | Ast.MetaListlenDecl(r,n) -> print_name rule r n
+  | Ast.MetaListlenDecl(r,n) -> ()
   | Ast.MetaParamDecl(_,(r,n)) ->
       print_string "parameter "; print_name rule r n; print_string ";"
   | Ast.MetaBinaryOperatorDecl(_,(r,n)) -> (* missing constraints *)
@@ -948,6 +940,10 @@ let rec unparse_cocci_mv rule = function
   | Ast.MetaExpDecl(_,(r,n),None) ->
       print_string "expression "; print_name rule r n; print_string ";"
   | Ast.MetaExpDecl(_,(r,n),ty) ->
+      (match ty with
+	None -> ()
+      | Some ty -> (* unknown only possible when there is only one type? *)
+	  if List.exists contains_unknown ty then print_string "expression ");
       print_types ty; print_name rule r n; print_string ";"
   | Ast.MetaIdExpDecl(_,(r,n),ty) ->
       print_string "idexpression "; print_types ty;
@@ -973,6 +969,9 @@ let rec unparse_cocci_mv rule = function
   | Ast.MetaStmListDecl(_,(r,n),len) ->
       print_string "statement list "; print_listlen rule len;
       print_name rule r n; print_string ";"
+  | Ast.MetaDParamListDecl(_,(r,n),len) ->
+      print_string "identifier list "; print_listlen rule len;
+      print_name rule r n; print_string ";"
   | Ast.MetaFuncDecl(_,(r,n)) ->
       print_string "function "; print_name rule r n; print_string ";"
   | Ast.MetaLocalFuncDecl(_,(r,n)) ->
@@ -1050,6 +1049,7 @@ let _ =
     | Ast.ParamDotsTag(x) -> parameter_list x
     | Ast.StmtDotsTag(x) -> dots (function _ -> ()) (statement "") x
     | Ast.AnnDeclDotsTag(x) -> dots (function _ -> ()) (annotated_decl "") x
+    | Ast.DefParDotsTag(x) -> dots (function _ -> ()) print_define_param x
     | Ast.TypeCTag(x) -> typeC x
     | Ast.ParamTag(x) -> parameterTypeDef x
     | Ast.SgrepStartTag(x) -> print_string x
@@ -1070,6 +1070,8 @@ let rec dep in_and = function
       if not in_and
       then print_or ()
       else (print_string "("; print_or(); print_string ")")
+  | Ast.FileIn s -> print_string "file in "; print_string s
+  | Ast.NotFileIn s -> print_string "not file in "; print_string s
   | Ast.NoDep   -> print_string "no_dep"
   | Ast.FailDep -> print_string "fail_dep"
 
