@@ -408,10 +408,11 @@ and contains_only_minus =
     | _ -> k e in
 
   V0.flat_combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    dots dots dots dots dots dots
-    identifier expression donothing donothing typeC donothing donothing declaration
-    statement donothing case_line donothing donothing
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode
+    dots dots dots dots dots dots dots
+    identifier expression donothing donothing typeC donothing donothing
+    declaration statement donothing case_line donothing donothing
 
 
 (* needs a special case when there is a Disj or an empty DOTS *)
@@ -500,7 +501,7 @@ let all_minus s =
     Ast0.MINUS(_) -> true
   | _ -> false
 
-let rec unchanged_minus s =
+let unchanged_minus s =
   match Ast0.get_mcodekind s with
     Ast0.MINUS(mc) ->
       (match !mc with (Ast.NOREPLACEMENT,_) -> true | _ -> false)
