@@ -22,6 +22,7 @@ let visitor mode bind option_default
     fix_mcode unary_mcode arithOp_mcode logicalOp_mcode cv_mcode sign_mcode
     struct_mcode storage_mcode inc_mcode
     dotsexprfn dotsinitfn dotsparamfn dotsstmtfn dotsdeclfn dotscasefn
+    dotsdefparfn
     identfn exprfn assignOpfn binaryOpfn tyfn initfn paramfn declfn stmtfn
     forinfofn casefn string_fragmentfn topfn =
   let multibind l =
@@ -66,6 +67,7 @@ let visitor mode bind option_default
   and case_line_dots d = dotsfn dotscasefn case_line all_functions d
   and string_fragment_dots d = dotsfn strdotsfn string_fragment all_functions d
   and exec_code_dots d = dotsfn ecdotsfn exec_code all_functions d
+  and define_param_dots d = dotsfn dotsdefparfn define_param all_functions d
 
   and ident i =
     let k i =
@@ -882,14 +884,14 @@ let visitor mode bind option_default
 	    (multibind [lp_n;params_n;rp_n], Ast0.DParams(lp,params,rp))) in
     k p
 
-  and define_param_dots d =
-    rewrap d (map_split_bind define_param (Ast0.unwrap d))
-
   and define_param p =
     let k p =
       rewrap p
 	(match Ast0.unwrap p with
 	  Ast0.DParam(id) -> let (n,id) = ident id in (n,Ast0.DParam(id))
+	| Ast0.MetaDParamList(name,lenname,pure) ->
+	    let (n,name) = meta_mcode name in
+	    (n,Ast0.MetaDParamList(name,lenname,pure))
 	| Ast0.DPComma(comma) ->
 	    let (n,comma) = string_mcode comma in (n,Ast0.DPComma(comma))
 	| Ast0.DPdots(d) ->
@@ -1021,6 +1023,9 @@ let visitor mode bind option_default
       | Ast0.DotsCaseTag(cases) ->
 	  let (cases_n,cases) = case_line_dots cases in
 	  (cases_n,Ast0.DotsCaseTag(cases))
+      | Ast0.DotsDefParamTag(dps) ->
+	  let (dps_n,dps) = define_param_dots dps in
+	  (dps_n,Ast0.DotsDefParamTag(dps))
       | Ast0.IdentTag(id) ->
 	  let (id_n,id) = ident id in
 	  (id_n,Ast0.IdentTag(id))
@@ -1073,7 +1078,9 @@ let visitor mode bind option_default
       | Ast0.IsoWhenFTag(e) ->
 	  let (e_n,e) = expression e in
 	  (e_n,Ast0.IsoWhenFTag(e))
-      |	Ast0.MetaPosTag(var) -> failwith "not supported"
+      | Ast0.MetaPosTag(Ast0.MetaPos(nm,constraints,ct)) ->
+       	 let (n,nm) = meta_mcode nm in
+	   (n,Ast0.MetaPosTag(Ast0.MetaPos(nm,constraints,ct)))
       |	Ast0.HiddenVarTag(var) -> failwith "not supported"
       |	Ast0.WhenTag(a,e,b) -> anything b in
     k a
@@ -1082,7 +1089,8 @@ let visitor mode bind option_default
      represented elsewhere in the code *)
 
   and all_functions =
-    {VT0.ident = ident;
+    {VT0.meta_mcode = meta_mcode;
+      VT0.ident = ident;
       VT0.expression = expression;
       VT0.assignOp = assignOp;
       VT0.binaryOp = binaryOp;
@@ -1095,12 +1103,14 @@ let visitor mode bind option_default
       VT0.statement = statement;
       VT0.forinfo = forinfo;
       VT0.case_line = case_line;
+      VT0.define_param = define_param;
       VT0.string_fragment = string_fragment;
       VT0.top_level = top_level;
       VT0.expression_dots = expression_dots;
       VT0.statement_dots = statement_dots;
       VT0.declaration_dots = declaration_dots;
       VT0.case_line_dots = case_line_dots;
+      VT0.define_param_dots = define_param_dots;
       VT0.anything = anything} in
   all_functions
 
@@ -1125,6 +1135,7 @@ let combiner_functions =
    VT0.combiner_dotsstmtfn = (fun r k e -> k e);
    VT0.combiner_dotsdeclfn = (fun r k e -> k e);
    VT0.combiner_dotscasefn = (fun r k e -> k e);
+   VT0.combiner_dotsdefparfn = (fun r k e -> k e);
    VT0.combiner_identfn = (fun r k e -> k e);
    VT0.combiner_exprfn = (fun r k e -> k e);
    VT0.combiner_assignOpfn = (fun r k e -> k e);
@@ -1140,7 +1151,9 @@ let combiner_functions =
    VT0.combiner_topfn = (fun r k e -> k e)}
 
 let combiner_dz r =
-    {VT0.combiner_rec_ident =
+    {VT0.combiner_rec_meta_mcode =
+      (function e -> let (n,_) = r.VT0.meta_mcode e in n);
+      VT0.combiner_rec_ident =
       (function e -> let (n,_) = r.VT0.ident e in n);
       VT0.combiner_rec_expression =
       (function e -> let (n,_) = r.VT0.expression e in n);
@@ -1166,6 +1179,8 @@ let combiner_dz r =
       (function e -> let (n,_) = r.VT0.forinfo e in n);
       VT0.combiner_rec_case_line =
       (function e -> let (n,_) = r.VT0.case_line e in n);
+      VT0.combiner_rec_define_param =
+      (function e -> let (n,_) = r.VT0.define_param e in n);
       VT0.combiner_rec_string_fragment =
       (function e -> let (n,_) = r.VT0.string_fragment e in n);
       VT0.combiner_rec_top_level =
@@ -1178,6 +1193,8 @@ let combiner_dz r =
       (function e -> let (n,_) = r.VT0.declaration_dots e in n);
       VT0.combiner_rec_case_line_dots =
       (function e -> let (n,_) = r.VT0.case_line_dots e in n);
+      VT0.combiner_rec_define_param_dots =
+      (function e -> let (n,_) = r.VT0.define_param_dots e in n);
       VT0.combiner_rec_anything =
       (function e -> let (n,_) = r.VT0.anything e in n)}
 
@@ -1189,12 +1206,16 @@ let combiner bind option_default functions =
     (function mc -> (functions.VT0.combiner_meta_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_string_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_const_mcode option_default mc,mc))
-    (function mc -> (functions.VT0.combiner_simpleAssign_mcode option_default mc,mc))
-    (function mc -> (functions.VT0.combiner_opAssign_mcode option_default mc,mc))
+    (function mc ->
+      (functions.VT0.combiner_simpleAssign_mcode option_default mc,mc))
+    (function mc ->
+      (functions.VT0.combiner_opAssign_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_fix_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_unary_mcode option_default mc,mc))
-    (function mc -> (functions.VT0.combiner_arithOp_mcode option_default mc,mc))
-    (function mc -> (functions.VT0.combiner_logicalOp_mcode option_default mc,mc))
+    (function mc ->
+      (functions.VT0.combiner_arithOp_mcode option_default mc,mc))
+    (function mc ->
+      (functions.VT0.combiner_logicalOp_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_cv_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_sign_mcode option_default mc,mc))
     (function mc -> (functions.VT0.combiner_struct_mcode option_default mc,mc))
@@ -1207,6 +1228,7 @@ let combiner bind option_default functions =
     (fun r k e -> (functions.VT0.combiner_dotsstmtfn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_dotsdeclfn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_dotscasefn (dz r) (xk k) e, e))
+    (fun r k e -> (functions.VT0.combiner_dotsdefparfn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_identfn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_exprfn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_assignOpfn (dz r) (xk k) e, e))
@@ -1226,6 +1248,7 @@ let flat_combiner bind option_default
     fix_mcode unary_mcode arithOp_mcode logicalOp_mcode cv_mcode sign_mcode
     struct_mcode storage_mcode inc_mcode
     dotsexprfn dotsinitfn dotsparamfn dotsstmtfn dotsdeclfn dotscasefn
+    dotsdefparfn
     identfn exprfn assignOpfn binaryOpfn tyfn initfn paramfn declfn stmtfn
     forinfofn casefn string_fragmentfn topfn =
   let dz = combiner_dz in
@@ -1251,6 +1274,7 @@ let flat_combiner bind option_default
     (fun r k e -> (dotsstmtfn (dz r) (xk k) e, e))
     (fun r k e -> (dotsdeclfn (dz r) (xk k) e, e))
     (fun r k e -> (dotscasefn (dz r) (xk k) e, e))
+    (fun r k e -> (dotsdefparfn (dz r) (xk k) e, e))
     (fun r k e -> (identfn (dz r) (xk k) e, e))
     (fun r k e -> (exprfn (dz r) (xk k) e, e))
     (fun r k e -> (assignOpfn (dz r) (xk k) e, e))
@@ -1286,6 +1310,7 @@ let rebuilder_functions =
    VT0.rebuilder_dotsstmtfn = (fun r k e -> k e);
    VT0.rebuilder_dotsdeclfn = (fun r k e -> k e);
    VT0.rebuilder_dotscasefn = (fun r k e -> k e);
+   VT0.rebuilder_dotsdefparfn = (fun r k e -> k e);
    VT0.rebuilder_identfn = (fun r k e -> k e);
    VT0.rebuilder_exprfn = (fun r k e -> k e);
    VT0.rebuilder_assignOpfn = (fun r k e -> k e);
@@ -1301,7 +1326,9 @@ let rebuilder_functions =
    VT0.rebuilder_topfn = (fun r k e -> k e)}
 
 let rebuilder_dz r =
-  {VT0.rebuilder_rec_ident =
+  {VT0.rebuilder_rec_meta_mcode =
+      (function e -> let (_,e) = r.VT0.meta_mcode e in e);
+      VT0.rebuilder_rec_ident =
       (function e -> let (_,e) = r.VT0.ident e in e);
       VT0.rebuilder_rec_expression =
         (function e -> let (_,e) = r.VT0.expression e in e);
@@ -1339,6 +1366,8 @@ let rebuilder_dz r =
       (function e -> let (_,e) = r.VT0.declaration_dots e in e);
       VT0.rebuilder_rec_case_line_dots =
       (function e -> let (_,e) = r.VT0.case_line_dots e in e);
+      VT0.rebuilder_rec_define_param_dots =
+      (function e -> let (_,e) = r.VT0.define_param_dots e in e);
       VT0.rebuilder_rec_anything =
       (function e -> let (_,e) = r.VT0.anything e in e)}
 
@@ -1367,6 +1396,7 @@ let rebuilder functions =
     (fun r k e -> ((),functions.VT0.rebuilder_dotsstmtfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_dotsdeclfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_dotscasefn (dz r) (xk k) e))
+    (fun r k e -> ((),functions.VT0.rebuilder_dotsdefparfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_identfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_exprfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_assignOpfn (dz r) (xk k) e))
@@ -1388,6 +1418,7 @@ let flat_rebuilder
     arithOp_mcode logicalOp_mcode cv_mcode sign_mcode struct_mcode
     storage_mcode inc_mcode
     dotsexprfn dotsinitfn dotsparamfn dotsstmtfn dotsdeclfn dotscasefn
+    dotsdefparfn
     identfn exprfn assignOpfn arithOpfn tyfn initfn paramfn declfn stmtfn
     forinfofn casefn string_fragmentfn topfn =
   let dz = rebuilder_dz in
@@ -1414,6 +1445,7 @@ let flat_rebuilder
     (fun r k e -> ((),dotsstmtfn (dz r) (xk k) e))
     (fun r k e -> ((),dotsdeclfn (dz r) (xk k) e))
     (fun r k e -> ((),dotscasefn (dz r) (xk k) e))
+    (fun r k e -> ((),dotsdefparfn (dz r) (xk k) e))
     (fun r k e -> ((),identfn (dz r) (xk k) e))
     (fun r k e -> ((),exprfn (dz r) (xk k) e))
     (fun r k e -> ((),assignOpfn (dz r) (xk k) e))
@@ -1463,6 +1495,7 @@ let combiner_rebuilder_functions =
    VT0.combiner_rebuilder_dotsstmtfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_dotsdeclfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_dotscasefn = (fun r k e -> k e);
+   VT0.combiner_rebuilder_dotsdefparfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_identfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_exprfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_assignOpfn = (fun r k e -> k e);
@@ -1499,6 +1532,7 @@ let combiner_rebuilder bind option_default functions =
     functions.VT0.combiner_rebuilder_dotsstmtfn
     functions.VT0.combiner_rebuilder_dotsdeclfn
     functions.VT0.combiner_rebuilder_dotscasefn
+    functions.VT0.combiner_rebuilder_dotsdefparfn
     functions.VT0.combiner_rebuilder_identfn
     functions.VT0.combiner_rebuilder_exprfn
     functions.VT0.combiner_rebuilder_assignOpfn
