@@ -6,7 +6,6 @@
 
 module Ast = Ast_cocci
 module Ast0 = Ast0_cocci
-module TC = Type_cocci
 module V0 = Visitor_ast0
 module VT0 = Visitor_ast0_types
 module S = Ast_tostring
@@ -114,7 +113,7 @@ let type_c ~form =
     | Ast.GlobalID -> ("global idexpression ", "global idexpression ")
     | Ast.CONST -> ("constant ", "constant ") in
   let type2c a =
-    match TC.type2c a with
+    match Ast.string_of_fullType a with
     | "unknown *" -> default ^ " *"
     | a -> prefix ^ a in
   function
@@ -133,42 +132,43 @@ let seed ~rn =
     | Ast.StringSeed s -> " = \"" ^ s ^ "\""
     | Ast.ListSeed s -> " = " ^ (String.concat " ## " (List.map se s))
 
-let regex_constraint = function
-  | Ast.IdRegExp (s,r) -> " =~ \"" ^ s ^ "\""
-  | Ast.IdNotRegExp (s,r) -> " !~ \"" ^ s ^"\""
-
 let list_constraints ~tostring_fn ~op = function
   | [] -> ""
   | [x] -> op ^ (tostring_fn x)
   | x -> op ^ "{" ^ (String.concat "," (List.map tostring_fn x)) ^ "}"
 
-let id_constraint ~rn =
-  let list_constraints' slist mnlist op =
-    let combined =
-      (List.map (fun x -> "\"" ^ x ^ "\"") slist) @
-      (List.map (name_str ~rn) mnlist) in
-    list_constraints ~tostring_fn:(fun x -> x) ~op combined in
+let general_constraint ~rn =
+  let list_constraints' list op =
+    list_constraints ~tostring_fn:(function
+	Ast.CstrString s -> "\"" ^ s ^ "\""
+      | Ast.CstrMeta_name mn -> name_str ~rn mn
+      | _ -> assert false) ~op list in
   function
-  | Ast.IdNoConstraint -> ""
-  | Ast.IdPosIdSet(slist,mnlist) -> list_constraints' slist mnlist " = "
-  | Ast.IdNegIdSet(slist,mnlist) -> list_constraints' slist mnlist " != "
-  | Ast.IdRegExpConstraint(re) -> regex_constraint re
+    Ast.CstrTrue -> "/* No constraint */"
+  | Ast.CstrRegexp (s,r) -> " =~ \"" ^ s ^ "\""
+  | Ast.CstrNot (Ast.CstrRegexp (s,r)) -> " !~ \"" ^ s ^"\""
+  | Ast.CstrOr list when
+      List.for_all
+	(function Ast.CstrString _  | Ast.CstrMeta_name _  -> true | _ -> false)
+	list ->
+	  list_constraints' list " = "
+  | Ast.CstrNot (Ast.CstrOr list) when
+      List.for_all
+	(function Ast.CstrString _  | Ast.CstrMeta_name _  -> true | _ -> false)
+	list ->
+	  list_constraints' list " != "
+  | Ast.CstrScript (_, lang, params, code) ->
+      Printf.sprintf ": script:%s (%s) { %s }" lang
+	(String.concat ","
+	   (List.map (fun (nm,_) -> name_str ~rn nm) params))
+	code
+  | _ -> ": ??"
 
-let id_constraint ~rn =
-  let list_constraints' slist mnlist op =
-    let combined =
-      (List.map (fun x -> "\"" ^ x ^ "\"") slist) @
-      (List.map (name_str ~rn) mnlist) in
-    list_constraints ~tostring_fn:(fun x -> x) ~op combined in
-  function
-  | Ast.IdNoConstraint -> ""
-  | Ast.IdPosIdSet(slist,mnlist) -> list_constraints' slist mnlist " = "
-  | Ast.IdNegIdSet(slist,mnlist) -> list_constraints' slist mnlist " != "
-  | Ast.IdRegExpConstraint(re) -> regex_constraint re
+let id_constraint = general_constraint
 
 let constraints ~rn = function
     Ast0.NoConstraint -> ""
-  | Ast0.NotIdCstrt recstr -> regex_constraint recstr
+  | Ast0.NotIdCstrt recstr -> general_constraint ~rn recstr
   | Ast0.NotExpCstrt exps ->
 
     (* exps is a list of expressions, but it is limited to numbers and ids
@@ -191,14 +191,18 @@ let constraints ~rn = function
       list_constraints ~tostring_fn:(name_str ~rn) ~op:" <= " mns
 
 let assign_constraints = function
-  | Ast0.AssignOpNoConstraint -> ""
-  | Ast0.AssignOpInSet l ->
-      list_constraints ~tostring_fn:Ast0.string_of_assignOp ~op:" = " l
+    Ast.CstrTrue -> ""
+  | Ast.CstrString s ->
+      list_constraints ~tostring_fn:Common.id ~op:" = " [s]
+  | Ast.CstrOr l when
+      List.for_all (function Ast.CstrString _ -> true | _ -> false) l ->
+      let get_string =
+	function Ast.CstrString s -> s | _ -> assert false in
+      let l' = List.map get_string l in
+      list_constraints ~tostring_fn:Common.id ~op:" = " l'
+  | _ -> failwith "assign_constraints: not implemented"
 
-let binary_constraints = function
-  | Ast0.BinaryOpNoConstraint -> ""
-  | Ast0.BinaryOpInSet l ->
-      list_constraints ~tostring_fn:Ast0.string_of_binaryOp ~op:" = " l
+let binary_constraints = assign_constraints
 
 let list_len ~rn = function
   | Ast0.AnyListLen -> " "
@@ -222,34 +226,43 @@ let types ~rn = function
   | Some typecs ->
       (* TODO: are the keep_bindings used for anything ? *)
       let bin = function
-        | TC.Unitary -> ""
-        | TC.Nonunitary -> ""
-        | TC.Saved -> "" in
-      let get_meta_id acc = function
-        | TC.MV(mn, b, _) ->
-            let metavar = make_mv "identifier" (name_tup ~rn mn) (bin b) in
+          Ast.Unitary -> ""
+        | Ast.Nonunitary -> ""
+        | Ast.Saved -> "" in
+      let get_meta_id acc ty =
+        match Ast_cocci.unwrap ty with
+          Ast_cocci.MetaId(mn, _, b, _) ->
+            let mn' = Ast_cocci.unwrap_mcode mn in
+            let metavar = make_mv "identifier" (name_tup ~rn mn') (bin b) in
             MVSet.add metavar acc
         | _ -> acc in
-      let rec get_meta_type acc = function
-        | TC.MetaType(mn, b, _) ->
-            let metavar = make_mv "type " (name_tup ~rn mn) (bin b) in
+      let rec get_meta_type acc ty =
+        match Ast_cocci.unwrap ty with
+          Ast.MetaType(mn, b, _) ->
+            let mn' = Ast_cocci.unwrap_mcode mn in
+            let metavar = make_mv "type " (name_tup ~rn mn') (bin b) in
             MVSet.add metavar acc
-        | TC.TypeName s ->
-            let metavar = ("typedef ", str_tup s, "") in
+        | Ast.TypeName s ->
+            let metavar = ("typedef ", str_tup (Ast.unwrap_mcode s), "") in
             MVSet.add metavar acc
-        | TC.Decimal(nm1, nm2) ->
-            MVSet.union (get_meta_id acc nm1) (get_meta_id acc nm2)
-        | TC.EnumName n
-        | TC.StructUnionName(_, n) ->
-            get_meta_id acc n
-        | TC.ConstVol (_, t)
-        | TC.SignedT (_, Some t)
-        | TC.Pointer t
-        | TC.FunctionPointer t
-        | TC.Array t ->
-            get_meta_type acc t
-        | _ -> acc in
-      List.fold_left get_meta_type MVSet.empty typecs
+        | Ast.Decimal(_, _, nm1, _, nm2, _) ->
+            let get_meta_id_opt v o = Common.default v (get_meta_id v) o in
+            let acc =
+              get_meta_id_opt acc (Ast_cocci.ident_of_expression_opt nm1) in
+            let acc =
+              get_meta_id_opt acc
+                (Common.default None Ast_cocci.ident_of_expression_opt nm2) in
+            acc
+        | Ast.EnumName (_, Some n)
+        | Ast.StructUnionName(_, Some n) -> get_meta_id acc n
+        | Ast.SignedT (_, Some t) -> get_meta_type acc t
+        | Ast.Pointer (t, _)
+        | Ast.FunctionPointer (t, _, _, _, _, _, _)
+        | Ast.Array (t, _, _, _) -> get_meta_type_full acc t
+        | _ -> acc
+      and get_meta_type_full acc ty =
+        get_meta_type acc (Common.just (Ast_cocci.typeC_of_fullType_opt ty)) in
+      List.fold_left get_meta_type_full MVSet.empty typecs
   | None -> MVSet.empty
 
 (* Function to call on mcodes. We are only interested in the mcodes because of
@@ -279,8 +292,8 @@ let mcode ~rn ~mc:(_,_,_,_,pos,_) =
     | Ast0.TypeCTag {Ast0.node = Ast0.MetaType((mn,_,_,_,p,_),_); _} ->
         handle_metavar ~typ:"type " ~mn ~positions:!p ~set
     | Ast0.MetaPosTag(Ast0.MetaPos((mn,_,_,_,_,_), mns, colt)) ->
-        let constr =
-          list_constraints ~tostring_fn:(name_str ~rn) ~op:" != " mns in
+        let oneconstr constr = general_constraint ~rn constr in
+	let constr = list_constraints ~tostring_fn:oneconstr ~op:"" [mns] in
         let collect = (match colt with Ast.PER -> "" | Ast.ALL -> " any") in
         let pos = make_mv "position " (name_tup ~rn mn) (constr ^ collect) in
         MVSet.add pos set
@@ -357,6 +370,7 @@ let metavar_combiner rn =
   let dotsstmtfn = donothing in
   let dotsdeclfn = donothing in
   let dotscasefn = donothing in
+  let dotsdefparfn = donothing in
   let forinfofn = donothing in
   let casefn = donothing in
   let topfn = donothing in
@@ -398,8 +412,8 @@ let metavar_combiner rn =
     match Ast0.unwrap v with
     | Ast0.MetaStmt (mc, pure) ->
         meta_mc_format ~mc ~typ:"statement " ~constr:""
-    | Ast0.MetaStmtList (mc, pure) ->
-        meta_mc_format ~mc ~typ:"statementlist[]" ~constr:""
+    | Ast0.MetaStmtList (mc, listlen, _) ->
+        lst_format ~mc ~typ:"statement list" ~listlen
     | Ast0.AsStmt (s1, s2)->
         let stmt = c.VT0.combiner_rec_statement in as_format s1 s2 stmt stmt
     | Ast0.Iterator (id, _, expdots, _, stmt,_) ->
@@ -416,13 +430,15 @@ let metavar_combiner rn =
         let constr = constraints ~rn constr in
         meta_mc_format ~mc ~typ:"error " ~constr
     | Ast0.MetaExpr (mc, constr, typeclist, form, _) ->
+        let typeclist' =
+          Common.map_option (List.map (Ast0toast.typeC false)) typeclist in
 
         (* types function finds metavariable types and identifiers that were
          * used in this expression and therefore need to be declared. *)
-        let types = types ~rn typeclist in
+        let types = types ~rn typeclist' in
 
         (* type_c function returns the types in pretty string format *)
-        let typ = type_c ~form typeclist in
+        let typ = type_c ~form typeclist' in
         let constr = constraints ~rn constr in
         MVSet.union (meta_mc_format ~mc ~typ ~constr) (types)
     | Ast0.MetaExprList (mc, listlen, _) ->
@@ -481,6 +497,15 @@ let metavar_combiner rn =
         as_format ptd ex par expr
     | _ -> fn v in
 
+(*
+  Not used for now, visiter not parameterized by this...
+  let define_paramfn c fn v =
+    match Ast0.unwrap v with
+    | Ast0.MetaDParamList(mc, listlen, pure) ->
+        lst_format ~mc ~typ:"identifier list" ~listlen
+    | _ -> fn v in
+*)
+
   let declfn c fn v =
     match Ast0.unwrap v with
     | Ast0.MetaDecl(mc, pure) ->
@@ -518,6 +543,7 @@ let metavar_combiner rn =
     fix_mcode unary_mcode arithOp_mcode logicalOp_mcode cv_mcode sign_mcode
     struct_mcode storage_mcode inc_mcode
     dotsexprfn dotsinitfn dotsparamfn dotsstmtfn dotsdeclfn dotscasefn
+    dotsdefparfn
     identfn exprfn assignOpfn binaryOpfn tyfn initfn paramfn declfn stmtfn
     forinfofn casefn string_fragmentfn topfn
 
@@ -534,8 +560,8 @@ let get_rule (_,(r,_),_) = r
 let get_name (_,(_,nm),_) = nm
 
 (* forces rule inheritance (except if rule is already inherited). *)
-let inherit_rule ~new_rule ((a,(b,c),d) as mv) =
-  if b = "" then (a,(new_rule,c),d) else mv
+let inherit_rule ?(force = false) ~new_rule ((a,(b,c),d) as mv) =
+  if b = "" || force then (a,(new_rule,c),d) else mv
 
 let print out mv = output_string out (tostring_mv mv)
 
